# Generated by Django 3.2.19 on 2023-05-08 16:39

from django.db import migrations, models

from vendor.fields import AutoSlugField


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0041_offer_billing_start_date"),
    ]

    operations = [
        migrations.AddField(
            model_name="offer",
            name="is_promotional",
            field=models.BooleanField(
                default=False,
                help_text="You can mark this offer as promotional to help identify it between normal priced and discount priced offers.",  # noqa: E501
                verbose_name="Is Promotional",
            ),
        ),
        migrations.AlterField(
            model_name="offer",
            name="slug",
            field=AutoSlugField(
                editable=True, populate_from="name", unique_with=("site__id",)
            ),
        ),
    ]



# Generated by Django 3.1.3 on 2021-01-07 23:09

# import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0017_auto_20201203_2107"),
    ]

    operations = [
        migrations.AddField(
            model_name="Address",
            name="uuid",
            field=models.UUIDField(
                editable=True, unique=False, verbose_name="UUID", null=True
            ),
        ),
        migrations.AddField(
            model_name="CustomerProfile",
            name="uuid",
            field=models.UUIDField(
                editable=True, unique=False, verbose_name="UUID", null=True
            ),
        ),
    ]



# Generated by Django 3.1.3 on 2020-11-05 15:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0007_offer_list_bundle_items"),
    ]

    operations = [
        migrations.AddField(
            model_name="offer",
            name="allow_multiple",
            field=models.BooleanField(
                default=False,
                help_text="Confirm the user wants to buy multiples of the product where typically there is just one purchased at a time.",  # noqa: E501
                verbose_name="Allow Multiple Purchase",
            ),
        ),
    ]



# Generated by Django 3.2.13 on 2022-06-09 16:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0036_post_payment_status_change"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="receipt",
            name="auto_renew",
        ),
        migrations.RemoveField(
            model_name="receipt",
            name="status",
        ),
        migrations.AlterField(
            model_name="offer",
            name="meta",
            field=models.JSONField(
                blank=True, default=dict, null=True, verbose_name="Meta"
            ),
        ),
        migrations.AlterField(
            model_name="payment",
            name="transaction",
            field=models.CharField(
                blank=True, max_length=80, null=True, verbose_name="Transaction ID"
            ),
        ),
        migrations.AlterField(
            model_name="receipt",
            name="meta",
            field=models.JSONField(
                blank=True, default=dict, null=True, verbose_name="Meta"
            ),
        ),
        migrations.AlterField(
            model_name="receipt",
            name="transaction",
            field=models.CharField(
                blank=True, max_length=80, null=True, verbose_name="Transaction"
            ),
        ),
        migrations.AlterField(
            model_name="receipt",
            name="vendor_notes",
            field=models.JSONField(
                blank=True, default=dict, null=True, verbose_name="Vendor Notes"
            ),
        ),
    ]



# Generated by Django 5.2.9 on 2025-12-22 17:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0044_site_settings_stripe_processor_rename"),
    ]

    operations = [
        migrations.AlterField(
            model_name="address",
            name="country",
            field=models.IntegerField(
                choices=[
                    (20, "Andorra"),
                    (784, "United Arab Emirates"),
                    (4, "Afghanistan"),
                    (28, "Antigua And Barbuda"),
                    (660, "Anguilla"),
                    (8, "Albania"),
                    (51, "Armenia"),
                    (530, "Netherlands Antilles"),
                    (24, "Angola"),
                    (10, "Antarctica"),
                    (32, "Argentina"),
                    (16, "American Samoa"),
                    (40, "Austria"),
                    (36, "Australia"),
                    (533, "Aruba"),
                    (31, "Azerbaijan"),
                    (70, "Bosnia And Herzegowina"),
                    (52, "Barbados"),
                    (50, "Bangladesh"),
                    (56, "Belgium"),
                    (854, "Burkina Faso"),
                    (100, "Bulgaria"),
                    (48, "Bahrain"),
                    (108, "Burundi"),
                    (204, "Benin"),
                    (60, "Bermuda"),
                    (96, "Brunei Darussalam"),
                    (68, "Bolivia"),
                    (76, "Brazil"),
                    (44, "Bahamas"),
                    (64, "Bhutan"),
                    (74, "Bouvet Island"),
                    (72, "Botswana"),
                    (112, "Belarus"),
                    (84, "Belize"),
                    (124, "Canada"),
                    (166, "Cocos (Keeling) Islands"),
                    (180, "The Democratic Republic Of The Congo"),
                    (140, "Central African Republic"),
                    (178, "Congo"),
                    (756, "Switzerland"),
                    (384, "Cote D'ivoire"),
                    (184, "Cook Islands"),
                    (152, "Chile"),
                    (120, "Cameroon"),
                    (156, "China"),
                    (170, "Colombia"),
                    (188, "Costa Rica"),
                    (192, "Cuba"),
                    (132, "Cape Verde"),
                    (162, "Christmas Island"),
                    (196, "Cyprus"),
                    (203, "Czech Republic"),
                    (276, "Germany"),
                    (262, "Djibouti"),
                    (840, "United States"),
                    (392, "Japan"),
                    (484, "Mexico"),
                ],
                default=840,
                verbose_name="Country/Region",
            ),
        ),
    ]



from django.db import models
from django.utils.translation import gettext_lazy as _

from vendor.config import DEFAULT_CURRENCY

from .choice import CURRENCY_CHOICES


#########
# PRICE
#########
class Price(models.Model):
    offer = models.ForeignKey(
        "vendor.Offer",
        verbose_name=_("Offer"),
        on_delete=models.CASCADE,
        related_name="prices",
    )
    cost = models.FloatField(_("Cost"), blank=True, null=True)
    currency = models.CharField(
        _("Currency"), max_length=4, choices=CURRENCY_CHOICES, default=DEFAULT_CURRENCY
    )
    start_date = models.DateTimeField(
        _("Start Date"), help_text=_("When should the price first become available?")
    )
    end_date = models.DateTimeField(
        _("End Date"),
        blank=True,
        null=True,
        help_text=_("When should the price expire?"),
    )
    priority = models.IntegerField(
        _("Priority"),
        help_text=_("Higher number takes priority"),
        blank=True,
        null=True,
    )

    def save(self, *args, **kwargs):
        if not self.priority:
            # TODO: Add check to see if this is the only price on the offer, then let it be 0.
            # If not, might need to do some assumptions to guess what it should be.
            self.priority = 0

        super().save(*args, **kwargs)

    class Meta:
        verbose_name = "Price"
        verbose_name_plural = "Prices"

    def __str__(self):
        return "{} for {}:{}".format(
            self.offer.name, dict(CURRENCY_CHOICES)[self.currency], self.cost
        )



import json
import logging

import stripe
from django import dispatch
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q, TextChoices
from django.http.response import HttpResponse
from django.utils import timezone
from django.utils.translation import gettext as _
from django.views import View
from django.views.decorators.csrf import csrf_exempt

from vendor.integrations import StripeIntegration
from vendor.models import CustomerProfile, Invoice, Subscription
from vendor.models.choice import InvoiceStatus, PurchaseStatus
from vendor.processors import StripeProcessor
from vendor.utils import get_site_from_request

logger = logging.getLogger(__name__)


##########
# SIGNALS
stripe_invoice_upcoming = dispatch.Signal()


class StripeEvents(TextChoices):
    INVOICE_PAID = "invoice.paid", _("Invoice Paid")
    INVOICE_PAYMENT_FAILED = "invoice.payment_failed", _("Invoice Payment Failed")
    INOVICE_PAYMENT_SUCCEEDED = "invoice.payment_succeeded", _(
        "Invoice Payment Succeeded"
    )
    INVOICE_UPCOMING = "invoice.upcoming", _("Upcoming Invoice")
    PAYMENT_INTENT_SUCCEDED = "payment_intent.succeeded", _("Payment Succeeded")
    PAYMENT_INTENT_PAYMENT_FAILED = "payment_intent.payment_failed", _(
        "Payment Intent Failed"
    )
    CHARGE_SUCCEEDED = "charge.succeeded", _("Charge Succeeded")
    CHARGE_REFUNDED = "charge.refunded", _("Charge Refunded")
    CHARGE_REFUND_UPDATED = "charge.refund.updated", _("Charge Refund Updated")
    SOURCE_EXPIRED = "customer.source.expired", _("Source Expired")
    CUSTOMER_UPDATED = "customer.updated", _("Customer Updated")
    SUBSCRIPTION_UPDATED = "customer.subscription.updated", _("Subscription Updated")
    SUBSCRIPTION_CREATED = "customer.subscription.created", _("Subscription Created")
    SUBSCRIPTION_DELETED = "customer.subscription.deleted", _("Subscription Deleted")
    SUBSCRIPTION_TRIAL_END = "customer.subscription.trial_will_end", _(
        "Trial Period Will End"
    )
    INVOICE_FINALIZED = "invoice.finalized", _("Invoice Finalized")
    INVOICE_PAYMENT_ACTION_REQUIRED = "invoice.payment_action_required", _(
        "Invoice Payment Action Required"
    )
    SETUP_INTENT_SUCCEEDED = "setup_intent.succeeded", _("Setup Intent Succeeded")
    CHARGE_DISPUTE_CREATED = "charge.dispute.created", _("Charge Dispute Created")
    CHARGE_DISPUTE_CLOSED = "charge.dispute.closed", _("Charge Dispute Closed")


class StripeBaseAPI(View):
    """Base webhook view that validates Stripe signatures and parses events."""

    def __init__(self, **kwargs):
        self.stripe_event = None  # Variable used to store the webhooks event data.

        super().__init__(**kwargs)

    @csrf_exempt
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)

    def is_valid_post(self, site):
        try:
            credentials = StripeIntegration(site)

            if credentials.instance and credentials.instance.private_key:
                self.event = stripe.Event.construct_from(
                    json.loads(self.request.body), credentials.instance.private_key
                )
            elif settings.STRIPE_PUBLIC_KEY:
                self.event = stripe.Event.construct_from(
                    json.loads(self.request.body), settings.STRIPE_PUBLIC_KEY
                )
            else:
                return False

        except ValueError as exce:
            logger.error(f"Stripe Webhook ValueError exception: {exce}")
            return False
        except Exception as exce:
            logger.error(f"Stripe Webhook exception: {exce}")
            return False

        return True

    def is_incoming_event_correct_and_recurring(self, event, desired_event):
        if event.type != desired_event:
            return False

        # This check is recuired to make sure that the event is related to a subscription.
        if self.event.data.object.billing_reason != "subscription_cycle":
            return False

        return True

    def is_incoming_event_correct(self, event, desired_event):
        if event.type != desired_event:
            return False

        return True


# ### Legacy Stripe APIs ### #


# Warning StripeSubscriptionInvoicePaid will removed in favor of StripeInvoicePaymentSuccededEvent
class StripeSubscriptionInvoicePaid(StripeBaseAPI):
    """Handle legacy subscription invoice.paid events and renew subscriptions."""

    def post(self, request, *args, **kwargs):
        stripe_invoice = self.event.data.object

        site = get_site_from_request(self.request)
        processor = StripeProcessor(site)

        if not self.is_valid_post(site):
            logger.error("StripeSubscriptionInvoicePaid error: invalid post")
            return HttpResponse(
                status=200, content="StripeSubscriptionInvoicePaid invalid post"
            )

        if not self.is_incoming_event_correct_and_recurring(
            self.event, StripeEvents.INVOICE_PAID
        ):
            logger.error(
                f"StripeSubscriptionInvoicePaid error: invalid event: {self.event}"
            )
            return HttpResponse(
                status=200,
                content=f"StripeSubscriptionInvoicePaid error: invalid event: {self.event}",
            )

        paid_date = timezone.datetime.fromtimestamp(
            stripe_invoice.status_transitions.paid_at, tz=timezone.utc
        )
        stripe_charge = processor.stripe_get_object(
            processor.stripe.Charge, stripe_invoice.charge
        )

        customer_profile, stripe_customer = (
            processor.get_customer_profile_and_stripe_customer(stripe_invoice.customer)
        )

        if not customer_profile:
            logger.error(
                f"StripeSubscriptionInvoicePaid error: email: {stripe_invoice.customer_email} does not exist"
            )
            return HttpResponse(
                status=200,
                content=f"StripeSubscriptionInvoicePaid error: email: {stripe_invoice.customer_email} does not exist",
            )

        stripe_subscription = processor.stripe_get_object(
            processor.stripe.Subscription, stripe_invoice.subscription
        )
        subscription, created = (
            processor.get_or_create_subscription_from_stripe_subscription(
                customer_profile, stripe_subscription
            )
        )

        if created:
            processor.sync_stripe_subscription(site, stripe_subscription)
            msg = f"Synced Stripe Subscription: ({subscription.pk},{stripe_subscription.id}), customer: ({customer_profile.pk}, {stripe_customer.id}) site: {site}"  # noqa: E501
            logger.info(msg)
            return HttpResponse(status=200, content=msg)

        stripe_product = processor.stripe_get_object(
            processor.stripe.Product, stripe_subscription.plan.product
        )
        offer = processor.get_offer_from_stripe_product(stripe_product)

        if not offer:
            logger.error(
                f"StripeSubscriptionInvoicePaid {subscription} has no offer attached"
            )
            return HttpResponse(
                status=200,
                content=f"StripeSubscriptionInvoicePaid {subscription} has no offer attached",
            )

        payment_status = processor.get_payment_status(
            stripe_charge.status, stripe_charge.refunded
        )
        processor.invoice, created = (
            processor.get_or_create_invoice_from_stripe_invoice(
                stripe_invoice, offer, customer_profile
            )
        )
        processor.renew_subscription(
            subscription,
            stripe_invoice.charge,
            payment_status,
            payment_success=True,
            submitted_date=paid_date,
        )

        return HttpResponse(status=200)


class StripeSubscriptionPaymentFailed(StripeBaseAPI):
    """Handle subscription invoice.payment_failed events and mark payment failure."""

    def post(self, request, *args, **kwargs):
        site = get_site_from_request(request)

        if not self.is_valid_post(site):
            logger.error("StripeSubscriptionPaymentFailed error: invalid post")
            return HttpResponse(
                status=200,
                content="StripeSubscriptionPaymentFailed error: invalid post",
            )

        if not self.is_incoming_event_correct_and_recurring(
            self.event, StripeEvents.INVOICE_PAYMENT_FAILED
        ):
            logger.error(
                f"StripeSubscriptionPaymentFailed error: invalid event: {self.event}"
            )
            return HttpResponse(
                status=200,
                content=f"StripeSubscriptionPaymentFailed error: invalid event: {self.event}",
            )

        stripe_invoice = self.event.data.object
        paid_date = timezone.datetime.fromtimestamp(
            stripe_invoice.effective_at, tz=timezone.utc
        )
        processor = StripeProcessor(site)

        customer_profile, stripe_customer = (
            processor.get_customer_profile_and_stripe_customer(stripe_invoice.customer)
        )
        if not customer_profile:
            logger.error(
                f"StripeSubscriptionPaymentFailed error: email: {stripe_invoice.customer_email} does not exist"
            )
            return HttpResponse(
                status=200,
                content=f"StripeSubscriptionPaymentFailed error: email: {stripe_invoice.customer_email} does not exist",  # noqa: E501
            )

        try:
            subscription = Subscription.objects.get(
                gateway_id=stripe_invoice.subscription, profile=customer_profile
            )
        except ObjectDoesNotExist:
            logger.error(
                f"StripeSubscriptionPaymentFailed customer: {customer_profile} does not have a subscription with stripe_id: {stripe_invoice.subscription}"  # noqa: E501
            )
            return HttpResponse(
                status=200,
                content=f"StripeSubscriptionPaymentFailed customer: {customer_profile} does not have a subscription with stripe_id: {stripe_invoice.subscription}",  # noqa: E501
            )

        offer = subscription.get_offer()

        if not offer:
            logger.error(
                f"StripeSubscriptionPaymentFailed {subscription} has no offer attached"
            )
            return HttpResponse(
                status=200,
                content=f"StripeSubscriptionPaymentFailed {subscription} has no offer attached",
            )

        invoice = Invoice.objects.create(
            profile=subscription.profile,
            site=subscription.profile.site,
            ordered_date=paid_date,
            status=InvoiceStatus.COMPLETE,
        )

        invoice.add_offer(offer)
        invoice.vendor_notes = {}
        invoice.vendor_notes["stripe_id"] = stripe_invoice.stripe_id
        invoice.save()
        processor.invoice = invoice
        processor.subscription_payment_failed(subscription, stripe_invoice.charge)

        return HttpResponse(status=200)


# Warning StripeInvoicePaid will removed in favor of StripeInvoicePaymentSuccededEvent
class StripeInvoicePaid(StripeBaseAPI):
    """Handle legacy invoice.payment_succeeded events for subscriptions."""

    def post(self, request, *args, **kwargs):
        site = get_site_from_request(self.request)
        processor = StripeProcessor(site)

        if not self.is_valid_post(site):
            logger.error("StripeInvoicePaid error: invalid post")
            return HttpResponse(
                status=200, content="StripeInvoicePaid error: invalid post"
            )

        if not self.is_incoming_event_correct(
            self.event, StripeEvents.INOVICE_PAYMENT_SUCCEEDED
        ):
            logger.error(f"StripeInvoicePaid error: invalid event {self.event}")
            return HttpResponse(
                status=200,
                content=f"StripeInvoicePaid error: invalid event {self.event}",
            )

        stripe_invoice = self.event.data.object

        customer_profile, stripe_customer = (
            processor.get_customer_profile_and_stripe_customer(stripe_invoice.customer)
        )

        try:
            customer_profile = CustomerProfile.objects.get(
                site=site, user__email=stripe_invoice.customer_email
            )
        except ObjectDoesNotExist:
            logger.error(
                f"StripeInvoicePaid error: email: {stripe_invoice.customer_email} does not exist"
            )
            return HttpResponse(
                status=200,
                content=f"StripeInvoicePaid error: email: {stripe_invoice.customer_email} does not exist",
            )

        if "stripe_id" not in customer_profile.meta:
            customer_profile.meta["stripe_id"] = stripe_invoice.customer
            customer_profile.save()

        try:
            subscription = Subscription.objects.get(
                gateway_id=stripe_invoice.subscription, profile=customer_profile
            )
        except ObjectDoesNotExist:
            logger.error(
                f"StripeInvoicePaid customer: {customer_profile} does not have a subscription with stripe_id: {stripe_invoice.subscription}"  # noqa: E501
            )
            return HttpResponse(
                status=200,
                content=f"StripeInvoicePaid customer: {customer_profile} does not have a subscription with stripe_id: {stripe_invoice.subscription}",  # noqa: E501
            )

        payment = subscription.payments.filter(
            Q(transaction="") | Q(transaction=None), status=PurchaseStatus.QUEUED
        ).first()
        receipt = subscription.receipts.filter(
            Q(transaction="") | Q(transaction=None)
        ).first()

        if not payment and not receipt:
            logger.warning(
                f"There are no payments to update for subscription: {subscription}. Stripe Invoice: {stripe_invoice.id}"  # noqa: E501
            )
            return HttpResponse(
                status=200,
                content=f"There are no payments to update for subscription: {subscription}. Stripe Invoice: {stripe_invoice.id}",  # noqa: E501
            )

        if payment:
            payment.transaction = stripe_invoice.charge
            payment.status = PurchaseStatus.SETTLED
            payment.save()

        if receipt:
            receipt.transaction = stripe_invoice.charge
            receipt.save()

        return HttpResponse(status=200)


class StripeCardExpiring(StripeBaseAPI):
    """Handle customer.source.expired events and notify via processor hooks."""

    def post(self, request, *args, **kwargs):
        site = get_site_from_request(self.request)

        if not self.is_valid_post(site):
            logger.error("StripeCardExpiring error: invalid event post")
            return HttpResponse(
                status=200, content="StripeCardExpiring error: invalid event post"
            )

        if not self.is_incoming_event_correct(self.event, StripeEvents.SOURCE_EXPIRED):
            logger.error(f"StripeCardExpiring error: invalid event {self.event}")
            return HttpResponse(
                status=200,
                content=f"StripeCardExpiring error: invalid event {self.event}",
            )

        stripe_card = self.event.data.object

        stripe_customer_id = stripe_card["customer"]
        if stripe_customer_id:
            try:
                customer_profile = CustomerProfile.objects.get(
                    meta__stripe_id=stripe_customer_id
                )
            except ObjectDoesNotExist:
                logger.error(
                    f"StripeCardExpiring: stripe id {stripe_customer_id} does not exist for customer in vendor"
                )
                return HttpResponse(
                    status=200,
                    content=f"StripeCardExpiring: stripe id {stripe_customer_id} does not exist for customer in vendor",  # noqa: E501
                )

            email = customer_profile.user.email
            logger.info(
                f"StripeCardExpiring: sending customer_source_expiring signal for site {site} and email {email}"
            )

            processor = StripeProcessor(site)
            processor.customer_card_expired(site, email)

        return HttpResponse(status=200)


class StripeSyncObjects(View):
    """Trigger a manual sync of Stripe objects into vendor models."""

    def get(self, request, *args, **kwargs):
        site = get_site_from_request(request)
        logger.info(f"StripeSyncObjects: started for site: {site}")

        processor = StripeProcessor(site)
        processor.sync_stripe_vendor_objects(site)

        logger.info(f"StripeSyncObjects: finished for site: {site}")
        return HttpResponse(status=200)


def process_stripe_invoice_line_items_payment_succeded(stripe_invoice, site):
    processor = StripeProcessor(site)

    customer_profile, stripe_customer = (
        processor.get_customer_profile_and_stripe_customer(stripe_invoice.customer)
    )

    stripe_charge = processor.stripe_get_object(
        processor.stripe.Charge, stripe_invoice.charge
    )
    stripe_payment_method = processor.stripe_get_object(
        processor.stripe.PaymentMethod, stripe_charge.payment_method
    )

    offers = processor.get_offers_from_invoice_line_items(stripe_invoice.lines["data"])

    for offer in offers:
        processor.invoice, created = (
            processor.get_or_create_invoice_from_stripe_invoice(
                stripe_invoice, offer, customer_profile
            )
        )

    payment = processor.get_or_create_payment_from_stripe_payment_and_charge(
        processor.invoice, stripe_payment_method, stripe_charge
    )

    if payment.status == PurchaseStatus.SETTLED:
        processor.create_single_purchase_receipts(
            processor.invoice, payment, stripe_charge
        )


def process_stripe_invoice_subscription_payment_succeded(stripe_invoice, site):
    processor = StripeProcessor(site)

    paid_date = timezone.datetime.fromtimestamp(
        stripe_invoice.status_transitions.paid_at, tz=timezone.utc
    )
    payment_status = PurchaseStatus.SETTLED

    if stripe_invoice.charge:
        stripe_charge = processor.stripe_get_object(
            processor.stripe.Charge, stripe_invoice.charge
        )
        payment_status = processor.get_payment_status(
            stripe_charge.status, stripe_charge.refunded
        )

    stripe_subscription = processor.stripe_get_object(
        processor.stripe.Subscription, stripe_invoice.subscription
    )
    subscription = processor.get_subscription(stripe_subscription)

    if not subscription:
        msg = f"Stripe Subscription Invoice was not processed: stripe_invoice: {stripe_invoice}"
        logger.error(msg)
        return HttpResponse(status=200, content=msg)

    stripe_product = processor.stripe_get_object(
        processor.stripe.Product, stripe_subscription.plan.product
    )
    offer = processor.get_offer_from_stripe_product(stripe_product)

    customer_profile, stripe_customer = (
        processor.get_customer_profile_and_stripe_customer(stripe_invoice.customer)
    )

    if not offer or not customer_profile:
        msg = f"Stripe Subscription Invoice was not processed, stripe_invoice: {stripe_invoice.id} offer: {offer}, customer_profile: {customer_profile} stripe_customer: {stripe_customer}"  # noqa: E501
        logger.error(msg)
        return HttpResponse(status=200, content=msg)

    processor.invoice, created = processor.get_or_create_invoice_from_stripe_invoice(
        stripe_invoice, offer, customer_profile
    )
    processor.renew_subscription(
        subscription,
        stripe_invoice.charge,
        payment_status,
        payment_success=True,
        submitted_date=paid_date,
    )

    # Remove any lingering payments.
    for payment in processor.invoice.payments.filter(transaction__in=[None, ""]):
        if receipt := payment.get_receipt():
            receipt.delete()
        payment.delete()

    return HttpResponse(
        status=200,
        content=f"Subscription: {stripe_subscription.id} renewed invoice: {processor.invoice.pk}",
    )


class StripeInvoicePaymentSuccededEvent(StripeBaseAPI):
    """Handle invoice.payment_succeeded events for line items and subscriptions."""

    def post(self, request, *args, **kwargs):
        site = get_site_from_request(self.request)

        if not self.is_valid_post(site):
            logger.error("StripeInvoicePaid error: invalid post")
            return HttpResponse(
                status=200, content="StripeInvoicePaid error: invalid post"
            )

        if not self.is_incoming_event_correct(
            self.event, StripeEvents.INOVICE_PAYMENT_SUCCEEDED
        ):
            logger.error(f"StripeInvoicePaid error: invalid event {self.event}")
            return HttpResponse(
                status=200,
                content=f"StripeInvoicePaid error: invalid event {self.event}",
            )

        stripe_invoice = self.event.data.object

        if "subscription" not in stripe_invoice or not stripe_invoice["subscription"]:
            # Line Items Invoice
            return process_stripe_invoice_line_items_payment_succeded(
                stripe_invoice, site
            )
        else:
            # Subscription Invoice
            return process_stripe_invoice_subscription_payment_succeded(
                stripe_invoice, site
            )


class StripeInvoiceUpcomingEvent(StripeBaseAPI):
    """Handle invoice.upcoming events and emit the invoice upcoming signal."""

    def post(self, request, *args, **kwargs):
        site = get_site_from_request(request)
        processor = StripeProcessor(site)

        if not self.is_valid_post(site):
            logger.error("StripeInvoicePaid error: invalid post")
            return HttpResponse(
                status=200, content="StripeInvoicePaid error: invalid post"
            )

        if not self.is_incoming_event_correct(
            self.event, StripeEvents.INVOICE_UPCOMING
        ):
            logger.error(f"StripeInvoicePaid error: invalid event {self.event}")
            return HttpResponse(
                status=200,
                content=f"StripeInvoicePaid error: invalid event {self.event}",
            )

        stripe_invoice = self.event.data.object
        customer_profile, stripe_customer = (
            processor.get_customer_profile_and_stripe_customer(stripe_invoice.customer)
        )
        if not customer_profile or not stripe_customer:
            logger.error(
                f"error retrieving customer information for request: {self.event}"
            )
            return HttpResponse(status=200)

        logger.info(
            f"Upcoming Invoice for stripe_customer: {stripe_customer} customer_profile: {customer_profile}"
        )

        stripe_invoice_upcoming.send(
            sender=self.__class__, customer_profile=customer_profile
        )



from django.urls import path

from vendor.api.stripe.elements.views import (
    StripeCreatePaymentIntent,
    StripeWebhookEventHandler,
)

app_name = "vendor_api"

urlpatterns = [
    # Stripe Specific API Endpoints
    # Payment Intents
    path(
        "payment-intent/",
        StripeCreatePaymentIntent.as_view(),
        name="stripe-payment-intent-create",
    ),
    # Webhook
    path(
        "webhook/",
        StripeWebhookEventHandler.as_view(),
        name="stripe-webhook",
    ),
]



import datetime as dt
from calendar import mdays
from decimal import ROUND_UP, Decimal

from django.contrib.sites.shortcuts import get_current_site
from django.core.exceptions import FieldDoesNotExist
from django.db.models import ForeignKey
from django.db.models.fields import DateField
from django.template.defaultfilters import slugify as django_slugify
from django.utils.encoding import force_str
from django.utils.functional import Promise
from django.utils.timezone import is_aware, localtime


#############
# Requst and Session Utils
def get_site_from_request(request):
    if hasattr(request, "site"):
        return request.site
    return get_current_site(request)


def get_or_create_session_cart(session):
    session_cart = {}
    if "session_cart" not in session:
        session["session_cart"] = session_cart
    session_cart = session.get("session_cart")

    return session_cart


def clear_session_purchase_data(request):
    if "billing_address_form" in request.session:
        del request.session["billing_address_form"]
    if "credit_card_form" in request.session:
        del request.session["credit_card_form"]


def get_future_date_months(today, add_months):
    """
    Returns a datetime object with the a new added months
    """
    newday = today.day
    newmonth = (((today.month - 1) + add_months) % 12) + 1
    newyear = today.year + (((today.month - 1) + add_months) // 12)
    if newday > mdays[newmonth]:
        newday = mdays[newmonth]
    if newyear % 4 == 0 and newmonth == 2:
        newday += 1
    return dt.datetime(newyear, newmonth, newday, tzinfo=dt.timezone.utc)


def get_future_date_days(today, add_days):
    """
    Returns a datetime object with the a new added days
    """
    return today + dt.timedelta(days=add_days)


def get_display_decimal(amount):
    return Decimal(amount).quantize(Decimal(".00"), rounding=ROUND_UP)


def force_str_if_proxy(value):
    """
    Convert Django lazy translation objects (Promise/__proxy__) to string, otherwise return as is.
    """
    if isinstance(value, Promise):
        return force_str(value)
    return value


# From django-autoslug
#  Copyright (c) 2018-present Justin Mayer
#  Copyright (c) 2008—2016 Andy Mikhailenko
#
#  This file is part of django-autoslug.
#
#  django-autoslug is free software under terms of the GNU Lesser
#  General Public License version 3 (LGPLv3) as published by the Free
#  Software Foundation. See the file README for copying conditions.
#


try:
    # i18n-friendly approach
    from unidecode import unidecode
except ImportError:
    try:
        # Cyrillic transliteration (primarily Russian)
        from pytils.translit import slugify
    except ImportError:
        # fall back to Django's default method
        slugify = django_slugify
else:
    # Use Django's default method over decoded string
    def slugify(value):
        return django_slugify(unidecode(value))


def get_prepopulated_value(field, instance):
    """
    Returns preliminary value based on `populate_from`.
    """
    if hasattr(field.populate_from, "__call__"):
        # AutoSlugField(populate_from=lambda instance: ...)
        return field.populate_from(instance)
    else:
        # AutoSlugField(populate_from='foo')
        attr = getattr(instance, field.populate_from)
        return callable(attr) and attr() or attr


def generate_unique_slug(field, instance, slug, manager):
    """
    Generates unique slug by adding a number to given value until no model
    instance can be found with such slug. If ``unique_with`` (a tuple of field
    names) was specified for the field, all these fields are included together
    in the query when looking for a "rival" model instance.
    """

    original_slug = slug = crop_slug(field, slug)

    default_lookups = tuple(get_uniqueness_lookups(field, instance, field.unique_with))

    index = 1

    if not manager:
        manager = field.model._default_manager

    # keep changing the slug until it is unique
    while True:
        # find instances with same slug
        lookups = dict(default_lookups, **{field.name: slug})
        rivals = manager.filter(**lookups)
        if instance.pk:
            rivals = rivals.exclude(pk=instance.pk)

        if not rivals:
            # the slug is unique, no model uses it
            return slug

        # the slug is not unique; change once more
        index += 1

        # ensure the resulting string is not too long
        tail_length = len(field.index_sep) + len(str(index))
        combined_length = len(original_slug) + tail_length
        if field.max_length < combined_length:
            original_slug = original_slug[: field.max_length - tail_length]

        # re-generate the slug
        data = dict(slug=original_slug, sep=field.index_sep, index=index)
        slug = "%(slug)s%(sep)s%(index)d" % data

        # ...next iteration...


def get_uniqueness_lookups(field, instance, unique_with):
    """
    Returns a dict'able tuple of lookups to ensure uniqueness of a slug.
    """
    for original_lookup_name in unique_with:
        if "__" in original_lookup_name:
            field_name, inner_lookup = original_lookup_name.split("__", 1)
        else:
            field_name, inner_lookup = original_lookup_name, None

        try:
            other_field = instance._meta.get_field(field_name)
        except FieldDoesNotExist:
            raise ValueError(
                "Could not find attribute %s.%s referenced"
                " by %s.%s (see constraint `unique_with`)"
                % (
                    instance._meta.object_name,
                    field_name,
                    instance._meta.object_name,
                    field.name,
                )
            )

        if field == other_field:
            raise ValueError(
                "Attribute %s.%s references itself in `unique_with`."
                ' Please use "unique=True" for this case.'
                % (instance._meta.object_name, field_name)
            )

        value = getattr(instance, field_name)
        if value is not False and not value:
            if other_field.blank:
                field_object = instance._meta.get_field(field_name)
                if isinstance(field_object, ForeignKey):
                    lookup = "%s__isnull" % field_name
                    yield lookup, True
                break
            raise ValueError(
                "Could not check uniqueness of %s.%s with"
                " respect to %s.%s because the latter is empty."
                ' Please ensure that "%s" is declared *after*'
                " all fields listed in unique_with."
                % (
                    instance._meta.object_name,
                    field.name,
                    instance._meta.object_name,
                    field_name,
                    field.name,
                )
            )
        if isinstance(value, dt.datetime) and is_aware(value):
            value = localtime(value)
        if isinstance(other_field, DateField):  # DateTimeField is a DateField subclass
            inner_lookup = inner_lookup or "day"

            if "__" in inner_lookup:
                raise ValueError(
                    "The `unique_with` constraint in %s.%s"
                    ' is set to "%s", but AutoSlugField only'
                    " accepts one level of nesting for dates"
                    ' (e.g. "date__month").'
                    % (instance._meta.object_name, field.name, original_lookup_name)
                )

            parts = ["year", "month", "day"]
            try:
                granularity = parts.index(inner_lookup) + 1
            except ValueError:
                raise ValueError(
                    'expected one of %s, got "%s" in "%s"'
                    % (parts, inner_lookup, original_lookup_name)
                )
            else:
                for part in parts[:granularity]:
                    lookup = f"{field_name}__{part}"
                    yield lookup, getattr(value, part)
        else:
            # TODO: this part should be documented as it involves recursion
            if inner_lookup:
                if not hasattr(value, "_meta"):
                    raise ValueError(
                        'Could not resolve lookup "%s" in `unique_with` of %s.%s'
                        % (original_lookup_name, instance._meta.object_name, field.name)
                    )
                for inner_name, inner_value in get_uniqueness_lookups(
                    field, value, [inner_lookup]
                ):
                    yield original_lookup_name, inner_value
            else:
                yield field_name, value


def crop_slug(field, slug):
    if field.max_length < len(slug):
        return slug[: field.max_length]
    return slug


try:
    import translitcodec  # noqa F401
except ImportError:
    pass
else:
    import re

    PUNCT_RE = re.compile(r'[\t !"#$%&\'()*\-/<=>?@\[\\\]^_`{|},.]+')

    def translitcodec_slugify(codec):
        def _slugify(value, delim="-", encoding=""):
            """
            Generates an ASCII-only slug.

            Borrowed from http://flask.pocoo.org/snippets/5/
            """
            if encoding:
                encoder = f"{codec}/{encoding}"
            else:
                encoder = codec
            result = []
            for word in PUNCT_RE.split(value.lower()):
                word = word.encode(encoder)
                if word:
                    result.append(word.decode())
            return str(delim.join(result))

        return _slugify

    translit_long = translitcodec_slugify("translit/long")
    translit_short = translitcodec_slugify("translit/short")
    translit_one = translitcodec_slugify("translit/one")



# Generated by Django 3.1 on 2020-09-30 20:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0004_offer_offer_description"),
    ]

    operations = [
        migrations.AddField(
            model_name="address",
            name="first_name",
            field=models.CharField(
                blank=True, max_length=150, verbose_name="First Name"
            ),
        ),
        migrations.AddField(
            model_name="address",
            name="last_name",
            field=models.CharField(
                blank=True, max_length=150, verbose_name="Last Name"
            ),
        ),
    ]



# Generated by Django 3.1.3 on 2021-09-13 20:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0023_profile_null_false"),
    ]

    operations = [
        migrations.AddField(
            model_name="offer",
            name="deleted",
            field=models.BooleanField(default=False, verbose_name="Deleted"),
        ),
    ]



# Generated by Django 3.1.3 on 2020-11-20 17:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0010_auto_20201112_0138"),
    ]

    operations = [
        migrations.AlterField(
            model_name="address",
            name="address_1",
            field=models.CharField(max_length=40, verbose_name="Address"),
        ),
        migrations.AlterField(
            model_name="address",
            name="address_2",
            field=models.CharField(
                blank=True,
                max_length=40,
                null=True,
                verbose_name="Address 2 (Optional)",
            ),
        ),
        migrations.AlterField(
            model_name="address",
            name="country",
            field=models.IntegerField(
                choices=[(581, "United States")],
                default=581,
                verbose_name="Country/Region",
            ),
        ),
    ]



# Generated by Django 3.1 on 2020-09-25 19:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0003_auto_20200915_1839"),
    ]

    operations = [
        migrations.AddField(
            model_name="offer",
            name="offer_description",
            field=models.TextField(
                blank=True, null=True, verbose_name="Offer Description"
            ),
        ),
    ]



# Generated by Django 3.1.3 on 2020-11-23 18:48

from django.db import migrations, models

import vendor.models.offer


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0011_auto_20201120_1750"),
    ]

    operations = [
        migrations.AlterField(
            model_name="offer",
            name="offer_description",
            field=models.TextField(
                blank=True,
                default=None,
                help_text="You can enter a list of descriptions. Note: if you inputs something here the product description will not show up.",  # noqa: E501
                null=True,
                verbose_name="Offer Description",
            ),
        ),
        migrations.AlterField(
            model_name="offer",
            name="term_details",
            field=models.JSONField(
                blank=True,
                default=vendor.models.offer.offer_term_details_default,
                help_text="term_units: 10/20(Day/Month), trial_occurrences: 1(defualt)",
                null=True,
                verbose_name="Term Details",
            ),
        ),
    ]



# Generated by Django 3.2.13 on 2022-06-01 22:17

from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.db import migrations

# from vendor.models import Payment


def post_payment_status_change(apps, schema_editor):
    ReceiptModel = apps.get_model("vendor", "Receipt")
    PaymentModel = apps.get_model("vendor", "Payment")

    for payment in PaymentModel.objects.all():
        try:
            receipt = ReceiptModel.objects.get(
                transaction=payment.transaction,
                created__year=payment.created.year,
                created__month=payment.created.month,
                created__day=payment.created.day,
            )
        except ObjectDoesNotExist:
            receipt = None
        except MultipleObjectsReturned:
            receipt = ReceiptModel.objects.filter(
                transaction=payment.transaction,
                created__year=payment.created.year,
                created__month=payment.created.month,
                created__day=payment.created.day,
            ).first()

        if receipt:
            r = ReceiptModel.objects.get(pk=receipt.pk)
            payment.status = r.status
            payment.save()
            print(f"Payment Status Updated {payment}")


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0035_add_subscription_relation"),
    ]

    operations = [
        migrations.RunPython(
            post_payment_status_change, reverse_code=migrations.RunPython.noop
        ),
    ]



# Generated by Django 3.1.14 on 2021-12-16 16:29

from django.db import migrations


def add_trial_day_key_offer_term_details(apps, schema_editor):
    OfferModel = apps.get_model("vendor", "Offer")
    for offer in OfferModel.objects.all():
        if "trial_days" not in offer.term_details:
            offer.term_details["trial_days"] = 0
            offer.save()


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0027_bugfix_vendor_notes_invoice_history"),
    ]

    operations = [
        migrations.RunPython(
            add_trial_day_key_offer_term_details, reverse_code=migrations.RunPython.noop
        ),
    ]



# Generated by Django 3.2.13 on 2022-06-07 17:12

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0034_subscription"),
    ]

    operations = [
        migrations.AddField(
            model_name="payment",
            name="subscription",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="payments",
                to="vendor.subscription",
                verbose_name="Subscription",
            ),
        ),
        migrations.AddField(
            model_name="receipt",
            name="subscription",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="receipts",
                to="vendor.subscription",
                verbose_name="Subscription",
            ),
        ),
    ]



# Generated by Django 3.2.13 on 2022-06-07 16:49

import uuid

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0033_payment_status"),
    ]

    operations = [
        migrations.CreateModel(
            name="Subscription",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="date created"
                    ),
                ),
                (
                    "updated",
                    models.DateTimeField(auto_now=True, verbose_name="last updated"),
                ),
                ("deleted", models.BooleanField(default=False, verbose_name="Deleted")),
                (
                    "uuid",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        unique=True,
                        verbose_name="UUID",
                    ),
                ),
                (
                    "gateway_id",
                    models.CharField(
                        max_length=80, verbose_name="Subscription Gateway ID"
                    ),
                ),
                (
                    "auto_renew",
                    models.BooleanField(default=False, verbose_name="Auto Renew"),
                ),
                (
                    "meta",
                    models.JSONField(
                        blank=True, default=dict, null=True, verbose_name="Meta"
                    ),
                ),
                (
                    "profile",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="subscriptions",
                        to="vendor.customerprofile",
                        verbose_name="Purchase Profile",
                    ),
                ),
                (
                    "status",
                    models.IntegerField(
                        choices=[
                            (10, "Pause"),
                            (20, "Active"),
                            (30, "Canceled"),
                            (40, "Suspended"),
                        ],
                        default=0,
                        verbose_name="Status",
                    ),
                ),
            ],
            options={
                "verbose_name": "Subscription",
                "verbose_name_plural": "Subscriptions",
            },
        ),
    ]



# Generated by Django 3.2.13 on 2022-06-01 22:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0032_alter_invoice_status"),
    ]

    operations = [
        migrations.AddField(
            model_name="payment",
            name="status",
            field=models.IntegerField(
                choices=[
                    (1, "Queued"),
                    (2, "Active"),
                    (10, "Authorized"),
                    (15, "Captured"),
                    (20, "Settled"),
                    (30, "Canceled"),
                    (35, "Refunded"),
                    (40, "Declined"),
                    (50, "Void"),
                ],
                default=0,
                verbose_name="Status",
            ),
        ),
    ]



#  Copyright (c) 2018-present Justin Mayer
#  Copyright (c) 2008—2016 Andy Mikhailenko
#
#  This file comes from django-autoslug to maintain compatibility
#  https://github.com/justinmayer/django-autoslug/
#
#  django-autoslug is free software under terms of the GNU Lesser
#  General Public License version 3 (LGPLv3) as published by the Free
#  Software Foundation. See the file README for copying conditions.
#
"""
Django settings that affect django-autoslug:

`AUTOSLUG_SLUGIFY_FUNCTION`
  Allows to define a custom slugifying function.

  The function can be repsesented as string or callable, e.g.::

      # custom function, path as string:
      AUTOSLUG_SLUGIFY_FUNCTION = 'some_app.slugify_func'

      # custom function, callable:
      AUTOSLUG_SLUGIFY_FUNCTION = some_app.slugify_func

      # custom function, defined inline:
      AUTOSLUG_SLUGIFY_FUNCTION = lambda slug: 'can i haz %s?' % slug

  If no value is given, default value is used.

  Default value is one of these depending on availability in given order:

  * `unidecode.unidecode()` if Unidecode_ is available;
  * `pytils.translit.slugify()` if pytils_ is available;
  * `django.template.defaultfilters.slugify()` bundled with Django.

  django-autoslug also ships a couple of slugify functions that use
  the translitcodec_ Python library, e.g.::

     # using as many characters as needed to make a natural replacement
     AUTOSLUG_SLUGIFY_FUNCTION = 'autoslug.utils.translit_long'

     # using the minimum number of characters to make a replacement
     AUTOSLUG_SLUGIFY_FUNCTION = 'autoslug.utils.translit_short'

     # only performing single character replacements
     AUTOSLUG_SLUGIFY_FUNCTION = 'autoslug.utils.translit_one'

.. _Unidecode: http://pypi.python.org/pypi/Unidecode
.. _pytils: http://pypi.python.org/pypi/pytils
.. _translitcodec: http://pypi.python.org/pypi/translitcodec

`AUTOSLUG_MODELTRANSLATION_ENABLE`
  Django-autoslug support of modeltranslation_ is still experimental.
  If you wish to enable it, please set this option to `True` in your project
  settings.  Default is `False`.

.. _modeltranslation: http://django-modeltranslation.readthedocs.org

"""
from django.conf import settings
from django.utils.module_loading import import_string

DEFAULT_SLUGIFY_FUNCTION = "django.utils.text.slugify"


def _get_setting(name, default):
    """
    Safe settings access that does not raise during import when Django is
    not configured (e.g., module-level imports in tests).
    """
    if settings.configured:
        return getattr(settings, name, default)
    return default


# use custom slugifying function if any
slugify_function_path = _get_setting(
    "AUTOSLUG_SLUGIFY_FUNCTION", DEFAULT_SLUGIFY_FUNCTION
)
slugify = import_string(slugify_function_path)

# enable/disable modeltranslation support
autoslug_modeltranslation_enable = _get_setting(
    "AUTOSLUG_MODELTRANSLATION_ENABLE", False
)



import json
import logging

import stripe
from django import dispatch
from django.conf import settings
from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import TextChoices
from django.http import JsonResponse
from django.http.response import HttpResponse
from django.shortcuts import get_object_or_404
from django.utils import timezone
from django.utils.translation import gettext as _
from django.views import View
from django.views.decorators.csrf import csrf_exempt

from vendor.integrations import StripeIntegration
from vendor.models import CustomerProfile, Invoice, Subscription
from vendor.models.choice import InvoiceStatus, PurchaseStatus
from vendor.processors import StripeProcessor
from vendor.utils import get_site_from_request

logger = logging.getLogger(__name__)


##########
# SIGNALS
stripe_invoice_upcoming = dispatch.Signal()


class StripeEvents(TextChoices):
    INVOICE_PAID = "invoice.paid", _("Invoice Paid")
    INVOICE_PAYMENT_FAILED = "invoice.payment_failed", _("Invoice Payment Failed")
    INOVICE_PAYMENT_SUCCEEDED = "invoice.payment_succeeded", _(
        "Invoice Payment Succeeded"
    )
    INVOICE_UPCOMING = "invoice.upcoming", _("Upcoming Invoice")
    PAYMENT_INTENT_SUCCEDED = "payment_intent.succeeded", _("Payment Succeeded")
    PAYMENT_INTENT_PAYMENT_FAILED = "payment_intent.payment_failed", _(
        "Payment Intent Failed"
    )
    CHARGE_SUCCEEDED = "charge.succeeded", _("Charge Succeeded")
    CHARGE_REFUNDED = "charge.refunded", _("Charge Refunded")
    CHARGE_REFUND_UPDATED = "charge.refund.updated", _("Charge Refund Updated")
    SOURCE_EXPIRED = "customer.source.expired", _("Source Expired")
    CUSTOMER_UPDATED = "customer.updated", _("Customer Updated")
    SUBSCRIPTION_UPDATED = "customer.subscription.updated", _("Subscription Updated")
    SUBSCRIPTION_CREATED = "customer.subscription.created", _("Subscription Created")
    SUBSCRIPTION_DELETED = "customer.subscription.deleted", _("Subscription Deleted")
    SUBSCRIPTION_TRIAL_END = "customer.subscription.trial_will_end", _(
        "Trial Period Will End"
    )
    INVOICE_FINALIZED = "invoice.finalized", _("Invoice Finalized")
    INVOICE_PAYMENT_ACTION_REQUIRED = "invoice.payment_action_required", _(
        "Invoice Payment Action Required"
    )
    SETUP_INTENT_SUCCEEDED = "setup_intent.succeeded", _("Setup Intent Succeeded")
    CHARGE_DISPUTE_CREATED = "charge.dispute.created", _("Charge Dispute Created")
    CHARGE_DISPUTE_CLOSED = "charge.dispute.closed", _("Charge Dispute Closed")


class StripeBaseAPI(View):
    """Base webhook view that validates Stripe signatures and parses events."""

    def __init__(self, **kwargs):
        self.stripe_event = None  # Variable used to store the webhooks event data.

        super().__init__(**kwargs)

    @csrf_exempt
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)

    def is_valid_post(self, site):
        try:
            credentials = StripeIntegration(site)

            if credentials.instance and credentials.instance.private_key:
                self.event = stripe.Event.construct_from(
                    json.loads(self.request.body), credentials.instance.private_key
                )
            elif settings.STRIPE_PUBLIC_KEY:
                self.event = stripe.Event.construct_from(
                    json.loads(self.request.body), settings.STRIPE_PUBLIC_KEY
                )
            else:
                return False

        except ValueError as exce:
            logger.error(f"Stripe Webhook ValueError exception: {exce}")
            return False
        except Exception as exce:
            logger.error(f"Stripe Webhook exception: {exce}")
            return False

        return True

    def is_incoming_event_correct_and_recurring(self, event, desired_event):
        if event.type != desired_event:
            return False

        # This check is recuired to make sure that the event is related to a subscription.
        if self.event.data.object.billing_reason != "subscription_cycle":
            return False

        return True

    def is_incoming_event_correct(self, event, desired_event):
        if event.type != desired_event:
            return False

        return True


# ### Stripe Elements Endpoints ###


class StripeWebhookEventHandler(StripeBaseAPI):
    """
    A single Stripe webhook handler that routes events to specific handlers.

    NOTE: Many of the handlers are stubs that log the event for future extension.
    The key implemented handlers reconcile invoice payments for one-time purchases
    and subscription renewals, as well as handling failed subscription payments.
    """

    def post(self, request, *args, **kwargs):
        self.site = get_site_from_request(self.request)
        self.processor = StripeProcessor(self.site)

        if not self.is_valid_post(self.site):
            logger.error("StripeWebhookEventHandler error: invalid post")
            return HttpResponse(
                status=200, content="StripeWebhookEventHandler error: invalid post"
            )

        event_type = self.event.type
        handler = self.get_event_handler(event_type)
        if not handler:
            logger.info(f"StripeWebhookEventHandler: unhandled event {event_type}")
            return HttpResponse(status=200)

        return handler()

    def get_event_handler(self, event_type):
        handlers = {
            StripeEvents.INOVICE_PAYMENT_SUCCEEDED: self.event_invoice_payment_succeeded,
            StripeEvents.INVOICE_UPCOMING: self.event_invoice_upcoming,
            StripeEvents.INVOICE_PAYMENT_FAILED: self.event_invoice_payment_failed,
            StripeEvents.INVOICE_FINALIZED: self.event_invoice_finalized,
            StripeEvents.INVOICE_PAYMENT_ACTION_REQUIRED: self.event_invoice_payment_action_required,
            StripeEvents.SOURCE_EXPIRED: self.event_source_expired,
            StripeEvents.PAYMENT_INTENT_SUCCEDED: self.event_payment_intent_succeeded,
            StripeEvents.PAYMENT_INTENT_PAYMENT_FAILED: self.event_payment_intent_failed,
            StripeEvents.CHARGE_SUCCEEDED: self.event_charge_succeeded,
            StripeEvents.CHARGE_REFUNDED: self.event_charge_refunded,
            StripeEvents.CHARGE_REFUND_UPDATED: self.event_charge_refund_updated,
            StripeEvents.CUSTOMER_UPDATED: self.event_customer_updated,
            StripeEvents.SUBSCRIPTION_UPDATED: self.event_subscription_updated,
            StripeEvents.SUBSCRIPTION_CREATED: self.event_subscription_created,
            StripeEvents.SUBSCRIPTION_DELETED: self.event_subscription_deleted,
            StripeEvents.SUBSCRIPTION_TRIAL_END: self.event_subscription_trial_will_end,
            StripeEvents.SETUP_INTENT_SUCCEEDED: self.event_setup_intent_succeeded,
            StripeEvents.CHARGE_DISPUTE_CREATED: self.event_charge_dispute_created,
            StripeEvents.CHARGE_DISPUTE_CLOSED: self.event_charge_dispute_closed,
        }

        return handlers.get(event_type, self.event_unhandled)

    def event_unhandled(self):
        event_type = getattr(self.event, "type", None)
        event_id = getattr(self.event, "id", None)
        logger.info(
            "StripeWebhookEventHandler: unhandled event %s id=%s",
            event_type,
            event_id,
        )
        return HttpResponse(status=200)

    def event_invoice_payment_succeeded(self):
        """
        Reconcile successful invoice payments into local records.

        Stripe sends this when an invoice is paid. We either create receipts for
        one-time purchases or renew subscriptions tied to the invoice.
        """
        stripe_invoice = self.event.data.object

        # Single Payment for One-Time Purchase (No Subscription) Success
        if "subscription" not in stripe_invoice or not stripe_invoice["subscription"]:
            return self._process_stripe_invoice_line_items_payment_succeded(
                stripe_invoice
            )

        # Subscription Payment Success
        return self._process_stripe_invoice_subscription_payment_succeded(
            stripe_invoice
        )

    def _process_stripe_invoice_line_items_payment_succeded(self, stripe_invoice):

        customer_profile, stripe_customer = (
            self.processor.get_customer_profile_and_stripe_customer(
                stripe_invoice.customer
            )
        )

        stripe_charge = self.processor.stripe_get_object(
            self.processor.stripe.Charge, stripe_invoice.charge
        )
        stripe_payment_method = self.processor.stripe_get_object(
            self.processor.stripe.PaymentMethod, stripe_charge.payment_method
        )

        offers = self.processor.get_offers_from_invoice_line_items(
            stripe_invoice.lines["data"]
        )

        for offer in offers:
            self.processor.invoice, created = (
                self.processor.get_or_create_invoice_from_stripe_invoice(
                    stripe_invoice, offer, customer_profile
                )
            )

        payment = self.processor.get_or_create_payment_from_stripe_payment_and_charge(
            self.processor.invoice, stripe_payment_method, stripe_charge
        )

        if payment.status == PurchaseStatus.SETTLED:
            self.processor.create_single_purchase_receipts(
                self.processor.invoice, payment, stripe_charge
            )

    def _process_stripe_invoice_subscription_payment_succeded(self, stripe_invoice):
        paid_date = timezone.datetime.fromtimestamp(
            stripe_invoice.status_transitions.paid_at, tz=timezone.utc
        )
        payment_status = PurchaseStatus.SETTLED

        if stripe_invoice.charge:
            stripe_charge = self.processor.stripe_get_object(
                self.processor.stripe.Charge, stripe_invoice.charge
            )
            payment_status = self.processor.get_payment_status(
                stripe_charge.status, stripe_charge.refunded
            )

        stripe_subscription = self.processor.stripe_get_object(
            self.processor.stripe.Subscription, stripe_invoice.subscription
        )
        subscription = self.processor.get_subscription(stripe_subscription)

        if not subscription:
            msg = f"Stripe Subscription Invoice was not processed: stripe_invoice: {stripe_invoice}"
            logger.error(msg)
            return HttpResponse(status=200, content=msg)

        stripe_product = self.processor.stripe_get_object(
            self.processor.stripe.Product, stripe_subscription.plan.product
        )
        offer = self.processor.get_offer_from_stripe_product(stripe_product)

        customer_profile, stripe_customer = (
            self.processor.get_customer_profile_and_stripe_customer(
                stripe_invoice.customer
            )
        )

        if not offer or not customer_profile:
            msg = f"Stripe Subscription Invoice was not processed, stripe_invoice: {stripe_invoice.id} offer: {offer}, customer_profile: {customer_profile} stripe_customer: {stripe_customer}"  # noqa: E501
            logger.error(msg)
            return HttpResponse(status=200, content=msg)

        self.processor.invoice, created = (
            self.processor.get_or_create_invoice_from_stripe_invoice(
                stripe_invoice, offer, customer_profile
            )
        )
        self.processor.renew_subscription(
            subscription,
            stripe_invoice.charge,
            payment_status,
            payment_success=True,
            submitted_date=paid_date,
        )

        # Remove any lingering payments.
        for payment in self.processor.invoice.payments.filter(
            transaction__in=[None, ""]
        ):
            if receipt := payment.get_receipt():
                receipt.delete()
            payment.delete()

        return HttpResponse(
            status=200,
            content=f"Subscription: {stripe_subscription.id} renewed invoice: {self.processor.invoice.pk}",
        )

    def event_invoice_upcoming(self):
        """
        Respond to Stripe's heads-up that a charge is about to happen.

        Stripe sends invoice.upcoming shortly before attempting to bill a
        subscription. We use it to resolve the customer profile and emit the
        `stripe_invoice_upcoming` signal so downstream code can notify the user
        or run pre-billing checks.
        """
        stripe_invoice = self.event.data.object

        customer_profile, stripe_customer = (
            self.processor.get_customer_profile_and_stripe_customer(
                stripe_invoice.customer
            )
        )
        if not customer_profile or not stripe_customer:
            logger.error(
                f"error retrieving customer information for request: {self.event}"
            )
            return HttpResponse(status=200)

        logger.info(
            f"Upcoming Invoice for stripe_customer: {stripe_customer} customer_profile: {customer_profile}"
        )

        stripe_invoice_upcoming.send(
            sender=self.__class__, customer_profile=customer_profile
        )
        return HttpResponse(status=200)

    def event_invoice_payment_failed(self):
        """
        Handle a failed subscription billing attempts from Stripe.

        Stripe sends invoice.payment_failed when a recurring charge cannot be
        collected. We create a local invoice tied to the offer so support or
        automated flows can detect the failure and follow up with the customer.
        """
        stripe_invoice = self.event.data.object
        paid_date = timezone.datetime.fromtimestamp(
            stripe_invoice.effective_at, tz=timezone.utc
        )

        customer_profile, stripe_customer = (
            self.processor.get_customer_profile_and_stripe_customer(
                stripe_invoice.customer
            )
        )
        if not customer_profile:
            logger.error(
                f"StripeWebhookEventHandler error: email: {stripe_invoice.customer_email} does not exist"
            )
            return HttpResponse(status=200)

        try:
            subscription = Subscription.objects.get(
                gateway_id=stripe_invoice.subscription, profile=customer_profile
            )
        except ObjectDoesNotExist:
            logger.error(
                f"StripeWebhookEventHandler customer: {customer_profile} does not have a subscription with stripe_id: {stripe_invoice.subscription}"  # noqa: E501
            )
            return HttpResponse(status=200)

        offer = subscription.get_offer()

        if not offer:
            logger.error(
                f"StripeWebhookEventHandler {subscription} has no offer attached"
            )
            return HttpResponse(status=200)

        invoice = Invoice.objects.create(
            profile=subscription.profile,
            site=subscription.profile.site,
            ordered_date=paid_date,
            status=InvoiceStatus.COMPLETE,
        )

        invoice.add_offer(offer)
        invoice.vendor_notes = {}
        invoice.vendor_notes["stripe_id"] = stripe_invoice.stripe_id
        invoice.save()
        self.processor.invoice = invoice
        self.processor.subscription_payment_failed(subscription, stripe_invoice.charge)

        return HttpResponse(status=200)

    def event_source_expired(self):
        """
        Handle expiring customer payment sources.

        Stripe emits customer.source.expired when a saved card is expiring or
        removed. We use it to find the local customer and trigger the processor
        hook that can notify them to update payment details.
        """
        stripe_card = self.event.data.object
        stripe_customer_id = stripe_card["customer"]
        if not stripe_customer_id:
            return HttpResponse(status=200)

        try:
            customer_profile = CustomerProfile.objects.get(
                meta__stripe_id=stripe_customer_id
            )
        except ObjectDoesNotExist:
            logger.error(
                f"StripeWebhookEventHandler: stripe id {stripe_customer_id} does not exist for customer in vendor"
            )
            return HttpResponse(status=200)

        email = customer_profile.user.email
        logger.info(
            f"StripeWebhookEventHandler: sending customer_source_expiring signal for site {self.site} and email {email}"  # noqa: E501
        )

        self.processor.customer_card_expired(self.site, email)

        return HttpResponse(status=200)

    def event_payment_intent_succeeded(self):
        """
        Acknowledge a completed PaymentIntent for one-off payments.

        This confirms Stripe captured funds successfully. We currently log the
        outcome and can extend it to reconcile local payments.
        """
        logger.info(
            f"StripeWebhookEventHandler: payment_intent.succeeded {self.event.data.object.id}"
        )
        return HttpResponse(status=200)

    def event_payment_intent_failed(self):
        """
        Capture failed PaymentIntent attempts for follow-up.

        Indicates the payment method failed. We log it and can later trigger a
        retry or customer notification flow.
        """
        logger.info(
            f"StripeWebhookEventHandler: payment_intent.payment_failed {self.event.data.object.id}"
        )
        return HttpResponse(status=200)

    def event_charge_succeeded(self):
        """
        Record successful charges outside invoice-driven flows.

        Useful for legacy or custom charge paths; we log and can reconcile later.
        """
        logger.info(
            f"StripeWebhookEventHandler: charge.succeeded {self.event.data.object.id}"
        )
        return HttpResponse(status=200)

    def event_charge_refunded(self):
        """
        Observe refunds to keep local records aligned.

        Stripe emits this on full or partial refunds. We log and can later
        update payment and access records.
        """
        logger.info(
            f"StripeWebhookEventHandler: charge.refunded {self.event.data.object.id}"
        )
        return HttpResponse(status=200)

    def event_charge_refund_updated(self):
        """
        Track updates to an existing refund.

        Used for refund status changes (partial or failed) and future syncing.
        """
        refund = self.event.data.object
        logger.info(f"StripeWebhookEventHandler: charge.refund.updated {refund.id}")
        return HttpResponse(status=200)

    def event_customer_updated(self):
        """
        Observe customer record changes from Stripe.

        This lets us sync email or metadata changes if needed. Currently we
        log for visibility and can extend to persist updates.
        """
        logger.info(
            f"StripeWebhookEventHandler: customer.updated {self.event.data.object.id}"
        )
        return HttpResponse(status=200)

    def event_subscription_created(self):
        """
        Create or sync a subscription when Stripe provisions it.

        Stripe emits this when a subscription is first created. We ensure a
        matching local Subscription exists so renewals and access tracking work.
        """
        stripe_subscription = self.event.data.object

        customer_profile, _stripe_customer = (
            self.processor.get_customer_profile_and_stripe_customer(
                stripe_subscription.customer
            )
        )
        if not customer_profile:
            logger.error(
                f"StripeWebhookEventHandler: customer not found for subscription {stripe_subscription.id}"
            )
            return HttpResponse(status=200)

        self.processor.get_or_create_subscription_from_stripe_subscription(
            customer_profile, stripe_subscription
        )
        return HttpResponse(status=200)

    def event_subscription_updated(self):
        """
        Sync subscription status changes from Stripe.

        This covers state transitions like trialing, past_due, or paused so the
        local subscription remains accurate for access control.
        """
        stripe_subscription = self.event.data.object

        customer_profile, _stripe_customer = (
            self.processor.get_customer_profile_and_stripe_customer(
                stripe_subscription.customer
            )
        )
        if not customer_profile:
            logger.error(
                f"StripeWebhookEventHandler: customer not found for subscription {stripe_subscription.id}"
            )
            return HttpResponse(status=200)

        subscription, _created = (
            self.processor.get_or_create_subscription_from_stripe_subscription(
                customer_profile, stripe_subscription
            )
        )
        if subscription:
            subscription.status = self.processor.get_subscription_status(
                stripe_subscription.status
            )
            subscription.save()

        return HttpResponse(status=200)

    def event_subscription_deleted(self):
        """
        Mark local subscriptions as canceled when Stripe removes them.

        Keeps local status aligned with Stripe-initiated cancellations.
        """
        stripe_subscription = self.event.data.object

        subscription = self.processor.get_subscription(stripe_subscription)
        if not subscription:
            logger.error(
                f"StripeWebhookEventHandler: subscription not found {stripe_subscription.id}"
            )
            return HttpResponse(status=200)

        subscription.status = self.processor.get_subscription_status("canceled")
        subscription.save()

        return HttpResponse(status=200)

    def event_invoice_finalized(self):
        """
        Note when Stripe finalizes an invoice.

        This indicates invoice totals are locked; useful for audits or alerts.
        """
        logger.info(
            f"StripeWebhookEventHandler: invoice.finalized {self.event.data.object.id}"
        )
        return HttpResponse(status=200)

    def event_invoice_payment_action_required(self):
        """
        Note when Stripe requires customer action to complete payment.

        Typically indicates SCA/3DS is needed; useful for prompting the user.
        """
        logger.info(
            f"StripeWebhookEventHandler: invoice.payment_action_required {self.event.data.object.id}"
        )
        return HttpResponse(status=200)

    def event_subscription_trial_will_end(self):
        """
        Warn that a subscription trial is nearing completion.

        Used to trigger reminders or upgrade prompts before billing begins.
        """
        stripe_subscription = self.event.data.object
        logger.info(
            f"StripeWebhookEventHandler: trial will end for subscription {stripe_subscription.id}"
        )
        return HttpResponse(status=200)

    def event_setup_intent_succeeded(self):
        """
        Confirm a payment method was saved for future use.

        Stripe emits this after a successful SetupIntent; useful for onboarding
        or enabling subscription starts.
        """
        logger.info(
            f"StripeWebhookEventHandler: setup_intent.succeeded {self.event.data.object.id}"
        )
        return HttpResponse(status=200)

    def event_charge_dispute_created(self):
        """
        Record that a chargeback/dispute was opened.

        Useful for alerting staff or restricting access pending resolution.
        """
        logger.info(
            f"StripeWebhookEventHandler: charge.dispute.created {self.event.data.object.id}"
        )
        return HttpResponse(status=200)

    def event_charge_dispute_closed(self):
        """
        Record dispute resolution.

        Used to clear alerts or restore access once resolved.
        """
        logger.info(
            f"StripeWebhookEventHandler: charge.dispute.closed {self.event.data.object.id}"
        )
        return HttpResponse(status=200)


class StripeCreatePaymentIntent(LoginRequiredMixin, View):
    """Create a Stripe PaymentIntent for the current invoice's one-time charges."""

    def post(self, request, *args, **kwargs):
        site = get_site_from_request(request)
        data = self._get_request_data(request)

        payment_method_id = data.get("payment_method_id")
        if not payment_method_id:
            return JsonResponse({"error": "payment_method_id is required"}, status=400)

        invoice = self._get_invoice(request, site, data.get("invoice_uuid"))
        if not invoice.order_items.exists():
            return JsonResponse({"error": "Invoice has no order items"}, status=400)

        amount = invoice.get_one_time_transaction_total()
        if amount <= 0:
            return JsonResponse(
                {"error": "Invoice has no one-time charges"}, status=400
            )

        processor = StripeProcessor(site, invoice)
        processor.validate_invoice_customer_in_stripe()
        processor.validate_invoice_offer_in_stripe()

        payment_intent_data = processor.build_payment_intent(
            amount, payment_method_id, currency=invoice.currency
        )
        payment_intent = processor.create_payment_intent(payment_intent_data)

        if not payment_intent:
            return JsonResponse(
                {"error": "Unable to create payment intent"}, status=400
            )

        return JsonResponse(
            {
                "payment_intent_id": payment_intent.id,
                "client_secret": payment_intent.client_secret,
                "amount": payment_intent.amount,
                "currency": payment_intent.currency,
            }
        )

    def _get_request_data(self, request):
        if request.body and "application/json" in request.content_type:
            try:
                return json.loads(request.body)
            except json.JSONDecodeError:
                return {}
        return request.POST

    def _get_invoice(self, request, site, invoice_uuid):
        if invoice_uuid:
            return get_object_or_404(
                Invoice,
                uuid=invoice_uuid,
                profile__user=request.user,
                profile__site=site,
            )

        profile, _ = request.user.customer_profile.get_or_create(site=site)
        return profile.get_cart_or_checkout_cart()



import logging

from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver

from vendor.config import PaymentProcessorSiteConfig, SupportedPaymentProcessor
from vendor.models import CustomerProfile, Offer
from vendor.processors import StripeProcessor, StripeQueryBuilder

logger = logging.getLogger(__name__)


@receiver(post_save, sender=CustomerProfile, dispatch_uid="stripe_customer_post_save")
def stripe_create_customer_signal(sender, instance, created, **kwargs):
    site_configured_processor = PaymentProcessorSiteConfig(instance.site)

    if (
        site_configured_processor.get_key_value("payment_processor")
        != SupportedPaymentProcessor.STRIPE.value
    ):
        logger.error(
            f"stripe_create_customer_signal instance: {instance.pk} was not created on stripe for site {instance.site} because site is not configured with Stripe"  # noqa: E501
        )
        return None

    if "stripe_id" in instance.meta:
        return None

    processor = StripeProcessor(instance.site)

    query_builder = StripeQueryBuilder()

    email_clause = query_builder.make_clause_template(
        field="email",
        value=instance.user.email,
        operator=query_builder.EXACT_MATCH,
        next_operator=query_builder.AND,
    )

    metadata_clause = query_builder.make_clause_template(
        field="metadata",
        key="site",
        value=instance.site.domain,
        operator=query_builder.EXACT_MATCH,
    )

    query = query_builder.build_search_query(
        processor.stripe.Customer, [email_clause, metadata_clause]
    )
    query_result = processor.stripe_query_object(processor.stripe.Customer, query)

    if not processor.transaction_succeeded:
        logger.error(f"stripe_create_customer_signal: {processor.transactions_info}")
        return None

    post_save.disconnect(
        sender=CustomerProfile, dispatch_uid="stripe_customer_post_save"
    )
    if query_result:
        logger.info(f"stripe_create_customer_signal: updating customer: {instance}")
        processor.update_stripe_customers([instance])
        if len(query_result.data) > 1:
            logger.warning(
                f"stripe_create_customer_signal: more than one customer found for email: {instance.user.email} on site: {instance.site.domain}"  # noqa: E501
            )

    processor.create_stripe_customers([instance])
    post_save.connect(
        receiver=stripe_create_customer_signal,
        sender=CustomerProfile,
        dispatch_uid="stripe_customer_post_save",
    )
    logger.info(
        f"stripe_create_customer_signal instance: {instance.pk} successfully created on Stripe"
    )


@receiver(
    post_delete, sender=CustomerProfile, dispatch_uid="stripe_customer_post_delete"
)
def stripe_delete_customer_signal(sender, instance, **kwargs):
    site_configured_processor = PaymentProcessorSiteConfig(instance.site)

    if (
        site_configured_processor.get_key_value("payment_processor")
        != SupportedPaymentProcessor.STRIPE.value
    ):
        logger.error(
            f"stripe_delete_customer_signal instance: {instance.pk} can't be deleted as this site: {instance.site} is not configured to use Stripe"  # noqa: E501
        )
        return None

    if "stripe_id" not in instance.meta:
        return None

    processor = StripeProcessor(instance.site)

    post_delete.disconnect(
        sender=CustomerProfile, dispatch_uid="stripe_customer_post_delete"
    )
    processor.stripe_delete_object(
        processor.stripe.Customer, instance.meta["stripe_id"]
    )
    post_delete.connect(
        receiver=stripe_delete_customer_signal,
        sender=CustomerProfile,
        dispatch_uid="stripe_customer_post_delete",
    )
    logger.info(
        f"stripe_delete_customer_signal instance: {instance.pk} was successfully deleted on Stripe"
    )


@receiver(post_save, sender=Offer, dispatch_uid="stripe_offer_post_save")
def stripe_create_offer_signal(sender, instance, created, **kwargs):
    site_configured_processor = PaymentProcessorSiteConfig(instance.site)

    if (
        site_configured_processor.get_key_value("payment_processor")
        != SupportedPaymentProcessor.STRIPE.value
    ):
        logger.error(
            f"stripe_create_offer_signal instance: {instance.pk} was not created on stripe for site {instance.site} because site is not configured with Stripe"  # noqa: E501
        )
        return None

    processor = StripeProcessor(instance.site)

    query_builder = StripeQueryBuilder()

    pk_clause = query_builder.make_clause_template(
        field="metadata",
        key="pk",
        value=str(instance.pk),
        operator=query_builder.EXACT_MATCH,
        next_operator=query_builder.AND,
    )

    metadata_clause = query_builder.make_clause_template(
        field="metadata",
        key="site",
        value=instance.site.domain,
        operator=query_builder.EXACT_MATCH,
    )

    query = query_builder.build_search_query(
        processor.stripe.Product, [pk_clause, metadata_clause]
    )
    query_result = processor.stripe_query_object(processor.stripe.Product, query)

    if not processor.transaction_succeeded:
        logger.error(f"stripe_create_offer_signal: {processor.transactions_info}")
        return None

    post_save.disconnect(sender=Offer, dispatch_uid="stripe_offer_post_save")
    if "stripe" not in instance.meta:
        instance.meta.update({"stripe": {"product_id": query_result["data"][0]["id"]}})
        instance.save()

    if query_result["data"]:
        logger.info(f"stripe_create_offer_signal: updating the offer: {instance}")
        processor.update_offers([instance])
    else:
        processor.create_offers([instance])

    post_save.connect(
        receiver=stripe_create_offer_signal,
        sender=Offer,
        dispatch_uid="stripe_offer_post_save",
    )
    logger.info(
        f"stripe_create_offer_signal instance: offer {instance.pk} successfully created on Stripe"
    )


@receiver(post_delete, sender=Offer, dispatch_uid="stripe_offer_post_delete")
def stripe_delete_offer_signal(sender, instance, **kwargs):
    site_configured_processor = PaymentProcessorSiteConfig(instance.site)

    if (
        site_configured_processor.get_key_value("payment_processor")
        != SupportedPaymentProcessor.STRIPE.value
    ):
        logger.error(
            f"stripe_delete_offer_signal instance: {instance.pk} can't be deleted as this site: {instance.site} is not configured to use Stripe"  # noqa: E501
        )
        return None

    if "stripe" not in instance.meta:
        return None

    processor = StripeProcessor(instance.site)

    post_delete.disconnect(sender=Offer, dispatch_uid="stripe_offer_post_delete")
    processor.stripe_delete_object(
        processor.stripe.Product, instance.meta["stripe"]["product_id"]
    )
    processor.stripe_update_object(
        processor.stripe.Price, instance.meta["stripe"]["price_id"], {"active": False}
    )

    if instance.meta.get("stripe").get("coupon_id"):
        processor.stripe_delete_object(
            processor.stripe.Coupon, instance.meta["stripe"]["coupon_id"]
        )
        logger.info(
            f"stripe_delete_offer_signal instance: offer {instance.pk} coupon was successfully deleted on Stripe"
        )

    post_delete.connect(
        receiver=stripe_delete_offer_signal,
        sender=Offer,
        dispatch_uid="stripe_offer_post_delete",
    )
    logger.info(
        f"stripe_delete_offer_signal instance: offer {instance.pk} was successfully deleted on Stripe"
    )



# Generated by Django 3.2.19 on 2023-07-12 16:40

from django.db import migrations


def rename_stripe_processor_site_settings(apps, schema_editor):
    SiteConfigModel = apps.get_model("siteconfigs", "SiteConfigModel")

    # Check if any records exist in the table
    if not SiteConfigModel.objects.exists():
        print("No records found in SiteConfigModel. Skipping migration.")
        return

    for site_setting in SiteConfigModel.objects.filter(
        key="vendor.config.PaymentProcessorSiteConfig",
        value__payment_processor="stripe_processor.StripeProcessor",
    ):
        site_setting.value["payment_processor"] = "stripe.StripeProcessor"
        site_setting.save()
        print(f"\nStripe Payment Processor Config Update for site {site_setting.site}")


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0043_invoice_global_discount"),
        ("siteconfigs", "0001_initial"),  # makes sure that siteconfigs app is loaded
    ]

    operations = [
        migrations.RunPython(
            rename_stripe_processor_site_settings,
            reverse_code=migrations.RunPython.noop,
        )
    ]



# Generated by Django 3.1.3 on 2020-11-12 01:38

import django.db.models.deletion
from django.db import migrations, models

import vendor.models.utils


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        ("vendor", "0009_auto_20201111_0743"),
    ]

    operations = [
        migrations.AlterField(
            model_name="customerprofile",
            name="site",
            field=models.ForeignKey(
                default=vendor.models.utils.set_default_site_id,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="customer_profile",
                to="sites.site",
                verbose_name="Site",
            ),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="site",
            field=models.ForeignKey(
                default=vendor.models.utils.set_default_site_id,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="invoices",
                to="sites.site",
                verbose_name="Site",
            ),
        ),
        migrations.AlterField(
            model_name="offer",
            name="site",
            field=models.ForeignKey(
                default=vendor.models.utils.set_default_site_id,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="product_offers",
                to="sites.site",
                verbose_name="Site",
            ),
        ),
        migrations.AlterField(
            model_name="payment",
            name="billing_address",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="vendor.address",
                verbose_name="Billing Address",
            ),
        ),
        migrations.AlterField(
            model_name="payment",
            name="created",
            field=models.DateTimeField(auto_now_add=True, verbose_name="Date Created"),
        ),
        migrations.AlterField(
            model_name="price",
            name="cost",
            field=models.FloatField(blank=True, null=True, verbose_name="Cost"),
        ),
        migrations.AlterField(
            model_name="price",
            name="offer",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="prices",
                to="vendor.offer",
                verbose_name="Offer",
            ),
        ),
        migrations.AlterField(
            model_name="receipt",
            name="meta",
            field=models.JSONField(default=dict, verbose_name="Meta"),
        ),
        migrations.AlterField(
            model_name="taxclassifier",
            name="taxable",
            field=models.BooleanField(verbose_name="Taxable"),
        ),
    ]



from django import template
from django.urls import reverse
from siteconfigs.models import SiteConfigModel

from vendor.config import (
    PaymentProcessorSiteConfig,
    StripeConnectAccountConfig,
    VendorSiteCommissionConfig,
)

register = template.Library()


@register.inclusion_tag("vendor/includes/edit_link.html")
def config_edit_link(config_key, config_pk):
    config = SiteConfigModel.objects.get(pk=config_pk)
    link = ""

    if config.key == PaymentProcessorSiteConfig().key:
        link = reverse(
            "vendor_admin:manager-config-processor-update", kwargs={"pk": config_pk}
        )
    elif config.key == StripeConnectAccountConfig().key:
        link = reverse(
            "vendor_admin:manager-config-stripe-connect-update",
            kwargs={"pk": config_pk},
        )
    elif config.key == VendorSiteCommissionConfig().key:
        link = reverse(
            "vendor_admin:manager-config-commission-update", kwargs={"pk": config_pk}
        )

    return {"link": link}



from django.urls import path

from vendor.views import vendor as vendor_views

app_name = "vendor"

urlpatterns = [
    path("", vendor_views.VendorHomeView.as_view(), name="vendor-home"),
    path("cart/", vendor_views.CartView.as_view(), name="cart"),
    path(
        "checkout/summary/<uuid:uuid>/",
        vendor_views.PaymentSummaryView.as_view(),
        name="purchase-summary",
    ),
    path(
        "checkout/account/",
        vendor_views.AccountInformationView.as_view(),
        name="checkout-account",
    ),
    path(
        "checkout/payment/", vendor_views.PaymentView.as_view(), name="checkout-payment"
    ),
    path(
        "checkout/review/",
        vendor_views.ReviewCheckoutView.as_view(),
        name="checkout-review",
    ),
    path(
        "customer/products/",
        vendor_views.ReceiptListView.as_view(),
        name="customer-products",
    ),
    path(
        "customer/product/<uuid:uuid>/receipt/",
        vendor_views.ReceiptDetailView.as_view(),
        name="customer-receipt",
    ),
    path(
        "customer/subscriptions/",
        vendor_views.SubscriptionsListView.as_view(),
        name="customer-subscriptions",
    ),
    path(
        "customer/subscription/update/<uuid:uuid>/payment/",
        vendor_views.SubscriptionUpdatePaymentView.as_view(),
        name="customer-subscription-update-payment",
    ),
    path(
        "customer/shipping/<uuid:uuid>/update/",
        vendor_views.ShippingAddressUpdateView.as_view(),
        name="customer-shipping-update",
    ),
    path(
        "customer/billing/<uuid:uuid>/update/",
        vendor_views.AddressUpdateView.as_view(),
        name="customer-billing-update",
    ),
    path(
        "transfer/existing-subscriptions/stripe/",
        vendor_views.TransferExistingSubscriptionsToStripe.as_view(),
        name="transfer-existing-subscription-to-stripe",
    ),
    # TODO: Add user's account mangement urls
    # TODO: add user's order details page
]



# Generated by Django 3.1.3 on 2021-01-07 23:10

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0019_fill_uuid_address_profile"),
    ]

    operations = [
        migrations.AlterField(
            model_name="Address",
            name="uuid",
            field=models.UUIDField(
                default=uuid.uuid4, editable=False, unique=True, verbose_name="UUID"
            ),
        ),
        migrations.AlterField(
            model_name="CustomerProfile",
            name="uuid",
            field=models.UUIDField(
                default=uuid.uuid4, editable=False, unique=True, verbose_name="UUID"
            ),
        ),
    ]



import uuid

from django.db import models
from django.db.models import Q
from django.urls import reverse
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from vendor.models.base import CreateUpdateModelBase, SoftDeleteModelBase
from vendor.models.choice import PurchaseStatus, SubscriptionStatus


class SubscriptionReportModelManger(models.Manager):
    def get_total_cancelled_subscriptions(self, site, start_date=None, end_date=None):
        qs = super().get_queryset()

        if not (start_date and end_date):
            return qs.filter(profile__site=site, status=SubscriptionStatus.CANCELED)

        elif start_date and end_date is None:
            return qs.filter(
                profile__site=site,
                status=SubscriptionStatus.CANCELED,
                updated__date__gte=start_date,
            )

        elif start_date is None and end_date:
            return qs.filter(
                profile__site=site,
                status=SubscriptionStatus.CANCELED,
                updated__date__lte=end_date,
            )

        return qs.filter(
            profile__site=site,
            status=SubscriptionStatus.CANCELED,
            updated__date__gte=start_date,
            updated__date__lte=end_date,
        )


class Subscription(SoftDeleteModelBase, CreateUpdateModelBase):
    """
    A link for all the purchases a user has made. Contains subscription start and end date.
    This is generated for each item a user purchases so it can be checked in other code.
    """

    uuid = models.UUIDField(
        _("UUID"),
        editable=False,
        unique=True,
        default=uuid.uuid4,
        null=False,
        blank=False,
    )
    gateway_id = models.CharField(_("Subscription Gateway ID"), max_length=80)
    profile = models.ForeignKey(
        "vendor.CustomerProfile",
        verbose_name=_("Purchase Profile"),
        on_delete=models.CASCADE,
        related_name="subscriptions",
    )
    auto_renew = models.BooleanField(_("Auto Renew"), default=False)
    status = models.IntegerField(
        _("Status"), choices=SubscriptionStatus.choices, default=0
    )
    meta = models.JSONField(_("Meta"), default=dict, blank=True, null=True)

    reports = SubscriptionReportModelManger()

    class Meta:
        verbose_name = "Subscription"
        verbose_name_plural = "Subscriptions"

    def __str__(self):
        if self.receipts.count():
            return f"{self.receipts.first().order_item.name}"

        return f"{self.uuid}"

    @property
    def name(self):
        return self.__str__()

    def get_absolute_url(self):
        return reverse("vendor:customer-receipt", kwargs={"uuid": self.uuid})

    def void(self):
        """
        Funtion to void (not cancel) a receipt by making the end_date now and disabling
        access to the customer to that given product. If the receipt is related to a
        subscription to a Payment Gateway, make sure to also cancel such subscription
        in the given Payment Gateway.
        """
        active_receipts = self.receipts.filter(end_date__gte=timezone.now())

        for receipt in active_receipts:
            receipt.end_date = timezone.now()
            receipt.meta["voided_on"] = receipt.end_date.strftime("%Y-%m-%d_%H:%M:%S")
            receipt.save()

        self.meta[receipt.end_date.strftime("%Y-%m-%d_%H:%M:%S")] = (
            f"voided receipst: {[receipt.uuid for receipt in active_receipts]}"
        )
        self.save()

    def cancel(self):
        self.status = SubscriptionStatus.CANCELED
        self.auto_renew = False
        self.meta[timezone.now().strftime("%Y-%m-%d_%H:%M:%S")] = (
            "Subscription Canceled"
        )
        self.save()

    def get_next_billing_date(self):
        """Returns the next billing date

        Function returns the next billing date which is the last receipt.end_date for the subscription.
        """
        receipts = self.receipts.filter(
            Q(deleted=False), Q(end_date__gte=timezone.now())
        ).order_by("end_date")

        if not receipts.count():
            return None

        return receipts.first().end_date

    def get_last_payment_date(self):
        """Return the payments related reciept start date.

        Instead of returning payments.create DateTime field it return the releated payments reciept.start_date
        because that field can be edited to match an actual payment date if necessary.

        Returns:
            None or DateTime
        """
        payment = (
            self.payments.filter(deleted=False, status__lte=PurchaseStatus.SETTLED)
            .order_by("-created")
            .first()
        )

        if not payment:
            return None

        receipt = payment.get_receipt()

        if receipt is None or not receipt.start_date:
            return None

        return receipt.start_date

    def is_on_trial(self):
        if not self.receipts.filter(deleted=False).count():
            return False

        trial_receipt = (
            self.receipts.filter(deleted=False, transaction__contains="trial")
            .order_by("start_date")
            .first()
        )

        if timezone.now() > trial_receipt.end_date:
            return False

        return True

    def get_total(self):
        return self.receipts.first().order_item.total

    def save_payment_info(self, payment_info):
        if "payment_info" not in self.meta:
            self.meta["payment_info"] = {}

        self.meta.update({"payment_info": payment_info})
        self.save()

    def get_offer(self):
        receipt = self.receipts.first()

        if not receipt:
            return None

        return receipt.order_item.offer



from django.core.exceptions import ObjectDoesNotExist  # noqa: F401
from django.utils.module_loading import import_string
from siteconfigs.models import SiteConfigModel  # noqa: F401

from vendor.config import PaymentProcessorSiteConfig
from vendor.processors.authorizenet import AuthorizeNetProcessor  # noqa: F401
from vendor.processors.base import PaymentProcessorBase  # noqa: F401
from vendor.processors.dummy import DummyProcessor  # noqa: F401
from vendor.processors.stripe import (  # noqa: F401
    PRORATION_BEHAVIOUR_CHOICE,
    StripeProcessor,
    StripeQueryBuilder,
)


def get_site_payment_processor(site):
    site_processor = PaymentProcessorSiteConfig(site)
    return import_string(
        f"vendor.processors.{site_processor.get_key_value('payment_processor')}"
    )



import logging
from datetime import datetime

from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.utils import timezone

from vendor.models import CustomerProfile, Offer, Subscription
from vendor.models.choice import SubscriptionStatus  # InvoiceStatus,
from vendor.processors import AuthorizeNetProcessor, StripeProcessor

logger = logging.getLogger(__name__)


def convert_authorize_net_submitted_date(submitted_date):
    try:
        submitted_datetime = datetime.strptime(
            submitted_date.pyval, "%Y-%m-%dT%H:%M:%S.%f%z"
        )

    except ValueError as exce:
        logger.error(f"sync_subscriptions_and_create_missing_receipts error {exce}")
        submitted_datetime = datetime.strptime(
            submitted_date.pyval, "%Y-%m-%dT%H:%M:%S%z"
        )

    return submitted_datetime


def get_last_transaction_datetime(transactions, authorize_net):
    if not transactions:
        return None

    if len(transactions) == 1:
        transaction_id = transactions[0].transId.text
        transaction_detail = authorize_net.get_transaction_detail(transaction_id)
        return convert_authorize_net_submitted_date(transaction_detail.submitTimeUTC)

    return convert_authorize_net_submitted_date(transactions[0].submitTimeUTC)


def transfer_subscriptions_from_authorizenet_to_stripe(site):
    authorize_net = AuthorizeNetProcessor(site)
    stripe = StripeProcessor(site)

    auth_subscriptions = authorize_net.get_list_of_subscriptions()

    for auth_subscription in auth_subscriptions:
        subscription_info = authorize_net.subscription_info(auth_subscription.id.text)
        logger.info(
            f"Syncing AuthorizeNet Subscription ID: {auth_subscription.id.text}"
        )
        subscription_transactions = authorize_net.get_subscription_transactions(
            subscription_info
        )
        last_transaction_datetime = get_last_transaction_datetime(
            subscription_transactions, authorize_net
        )

        if hasattr(subscription_info.subscription.profile, "email"):
            email = subscription_info.subscription.profile.email.text

            try:
                customer_profile = CustomerProfile.objects.get(
                    site=site, user__email__iexact=email
                )
                offers = Offer.objects.filter(
                    site=site, name__contains=subscription_info.subscription.name
                )
                stripe_customer = stripe.get_stripe_customer_from_email(email)

                if not stripe_customer:
                    stripe.create_stripe_customers([customer_profile])
                elif stripe_customer and "stripe_id" not in customer_profile.meta:
                    customer_profile.meta.update({"stripe_id": stripe_customer.id})
                    customer_profile.save()

                stripe_customer = stripe.stripe_get_object(
                    stripe.stripe.Customer,
                    customer_profile.meta["stripe_id"],
                    expand=["subscriptions"],
                )

                if "site" not in stripe_customer["metadata"]:
                    stripe.update_stripe_customers([customer_profile])

                if not offers.count():
                    raise ObjectDoesNotExist()
                offer = offers.first()

                invoice = customer_profile.get_cart()
                invoice.empty_cart()
                invoice.add_offer(offer)
                stripe.invoice = invoice

                for stripe_customer_subscription in stripe_customer.subscriptions:
                    stripe_product = stripe.stripe_get_object(
                        stripe.stripe.Product,
                        stripe_customer_subscription["plan"]["product"],
                    )
                    if (
                        stripe_product["metadata"]["pk"] == str(offer.pk)
                        and "stripe" not in offer.meta
                    ):
                        offer.meta.update({"stripe": {"product_id": stripe_product.id}})
                        offer.save()
                        stripe.sync_offer(offer)
                    if Subscription.objects.filter(
                        profile=customer_profile,
                        gateway_id=stripe_customer_subscription.id,
                    ).exists():
                        stripe.sync_stripe_subscription(
                            site, stripe_customer_subscription
                        )
                        raise Exception(
                            f"Stripe Subscription Synced {stripe_customer_subscription.id}"
                        )

                stripe_product = stripe.get_stripe_product(site, offer.pk)
                if not stripe_product:
                    stripe.create_offers([offer])

                if stripe_product and "stripe" not in offer.meta:
                    offer.meta.update({"stripe": {"product_id": stripe_product.id}})
                    offer.save()

                stripe.sync_offer_prices(offer)

                stripe_payment_methods = stripe.get_customer_payment_methods(
                    stripe_customer.id
                )
                if not stripe_payment_methods:
                    raise ObjectDoesNotExist(
                        f"stripe_payment_method does not exist for stripe_customer: {stripe_customer.id}"
                    )

                setup_intent_object = stripe.build_setup_intent(
                    stripe_payment_methods[0].id
                )
                stripe_setup_intent = stripe.stripe_create_object(
                    stripe.stripe.SetupIntent, setup_intent_object
                )
                if not stripe_setup_intent:
                    raise Exception(
                        f"Could not create stripe_setup_intent transaction_info: {stripe.transaction_info}"
                    )

                subscription_obj = stripe.build_subscription(
                    stripe.invoice.order_items.first(), stripe_payment_methods[0].id
                )
                trial_days = (
                    offer.get_offer_end_date(last_transaction_datetime) - timezone.now()
                )
                subscription_obj["trial_period_days"] = trial_days.days
                subscription_obj["billing_cycle_anchor"] = offer.get_offer_end_date(
                    last_transaction_datetime
                )
                stripe_subscription = stripe.stripe_create_object(
                    stripe.stripe.Subscription, subscription_obj
                )

                if (
                    not stripe_subscription
                    or stripe_subscription.status == "incomplete"
                ):
                    stripe.transaction_succeeded = False
                    logger.error(
                        f"AuthorizeNet Subscription {auth_subscription.id.text} was not transfered to stripe error {stripe.transaction_info}"  # noqa: E501
                    )
                else:
                    subscription = Subscription.objects.create(
                        gateway_id=stripe_subscription.id,
                        profile=customer_profile,
                        auto_renew=True,
                        status=SubscriptionStatus.ACTIVE,
                    )
                    subscription.meta["response"] = stripe.transaction_info
                    subscription.save()

            except ObjectDoesNotExist as exce:
                logger.exception(
                    f"sync_subscriptions exception: {exce} subscription: ({auth_subscription.id.text}, {subscription_info.subscription.name})"  # noqa: E501
                )
            except MultipleObjectsReturned as exce:
                logger.exception(
                    f"sync_subscriptions exception: {exce} subscription: ({auth_subscription.id.text}, {subscription_info.subscription.name})"  # noqa: E501
                )
            except Exception as exce:
                logger.exception(
                    f"sync_subscriptions exception: {exce} subscription: ({auth_subscription.id.text}, {subscription_info.subscription.name})"  # noqa: E501
                )



from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.sites.models import Site
from django.shortcuts import redirect, render
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.generic.edit import FormView, UpdateView
from django.views.generic.list import ListView
from siteconfigs.models import SiteConfigModel

from vendor.config import (
    PaymentProcessorForm,
    PaymentProcessorSiteConfig,
    StripeConnectAccountConfig,
    StripeConnectAccountForm,
    VendorSiteCommissionConfig,
    VendorSiteCommissionForm,
)


class PaymentProcessorSiteConfigsListView(LoginRequiredMixin, ListView):
    template_name = "vendor/manage/config_list.html"
    model = SiteConfigModel

    def get_queryset(self):
        payment_processor = PaymentProcessorSiteConfig()

        return SiteConfigModel.objects.filter(key=payment_processor.key)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        processor_config = PaymentProcessorSiteConfig()

        context["title"] = _("Payment Processors Configured")
        context["config_key"] = processor_config.key
        context["new_url"] = reverse("vendor_admin:manager-config-processor-create")

        return context


class PaymentProcessorCreateConfigView(LoginRequiredMixin, FormView):
    template_name = "vendor/manage/config.html"
    form_class = PaymentProcessorForm

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(**kwargs)
        processor_config = PaymentProcessorSiteConfig()
        form = self.get_form()

        existing_stripe_configs = SiteConfigModel.objects.filter(
            key=processor_config.key
        )
        sites = Site.objects.exclude(
            pk__in=[config.site.pk for config in existing_stripe_configs]
        )
        form.fields["site"].queryset = sites

        context["title"] = _("Payment Processor Config")
        context["form"] = form
        context["cancel_url"] = reverse("vendor_admin:manager-config-processor-list")

        return context

    def form_valid(self, form):
        site = form.cleaned_data["site"]

        processor_config = PaymentProcessorSiteConfig(site)
        processor_config.save(
            form.cleaned_data["payment_processor"], "payment_processor"
        )

        return redirect("vendor_admin:manager-config-processor-list")


class PaymentProcessorUpdateConfigView(LoginRequiredMixin, UpdateView):
    template_name = "vendor/manage/config.html"
    model = SiteConfigModel
    form_class = PaymentProcessorForm

    def get_form(self):
        return PaymentProcessorForm()

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        config_model = self.get_object()
        processor_config = PaymentProcessorSiteConfig(config_model.site)

        context["title"] = _("Edit Payment Processor Config")
        context["form"] = PaymentProcessorForm(
            initial={
                "site": config_model.site,
                "payment_processor": processor_config.get_key_value(
                    "payment_processor"
                ),
            }
        )
        context["form"].fields["site"].disabled = True
        context["cancel_url"] = reverse("vendor_admin:manager-config-processor-list")

        return context

    def post(self, request, *args, **kwargs):
        # Get the SiteConfigModel instance passed through the url
        self.object = self.get_object()

        form = self.get_form_class()(request.POST)
        # Make this site field required False since you can only update the account_number
        form.fields["site"].required = False

        if not form.is_valid():
            context = super().get_context_data(**kwargs)
            context["form"] = form
            return render(request, self.template_name, context)

        processor_config = PaymentProcessorSiteConfig(self.object.site)
        processor_config.save(
            form.cleaned_data["payment_processor"], "payment_processor"
        )

        return redirect("vendor_admin:manager-config-processor-list")


class StripeConnectAccountCreateConfigView(LoginRequiredMixin, FormView):
    template_name = "vendor/manage/config.html"
    form_class = StripeConnectAccountForm

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        stripe_config = StripeConnectAccountConfig()
        form = self.get_form()

        existing_stripe_configs = SiteConfigModel.objects.filter(key=stripe_config.key)
        sites = Site.objects.exclude(
            pk__in=[config.site.pk for config in existing_stripe_configs]
        )
        form.fields["site"].queryset = sites

        context["title"] = _("Stripe Connect Account")
        context["form"] = form
        context["cancel_url"] = reverse(
            "vendor_admin:manager-config-stripe-connect-list"
        )

        return context

    def form_valid(self, form):
        site = form.cleaned_data["site"]

        stripe_connect = StripeConnectAccountConfig(site)
        stripe_connect.save(
            form.cleaned_data["account_number"], "stripe_connect_account"
        )

        return redirect("vendor_admin:manager-config-stripe-connect-list")


class StripeConnectAccountConfigListView(LoginRequiredMixin, ListView):
    template_name = "vendor/manage/config_list.html"
    model = SiteConfigModel

    def get_queryset(self):
        stripe_config = StripeConnectAccountConfig()

        return self.model.objects.filter(key=stripe_config.key)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        stripe_config = StripeConnectAccountConfig()

        context["title"] = _("Stripe Connect Accounts")
        context["config_key"] = stripe_config.key
        context["new_url"] = reverse(
            "vendor_admin:manager-config-stripe-connect-create"
        )

        return context


class StripeConnectAccountUpdateConfigView(LoginRequiredMixin, UpdateView):
    template_name = "vendor/manage/config.html"
    model = SiteConfigModel
    form_class = StripeConnectAccountForm

    def get_form(self):
        return StripeConnectAccountForm()

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        config_model = self.get_object()
        stripe_config = StripeConnectAccountConfig(config_model.site)

        context["title"] = _("Stripe Connect Account")
        context["form"] = StripeConnectAccountForm(
            initial={
                "site": config_model.site,
                "account_number": stripe_config.get_key_value("stripe_connect_account"),
            }
        )
        context["form"].fields["site"].disabled = True
        context["cancel_url"] = reverse(
            "vendor_admin:manager-config-stripe-connect-list"
        )

        return context

    def post(self, request, *args, **kwargs):
        # Get the SiteConfigModel instance passed through the url
        self.object = self.get_object()

        form = self.get_form_class()(request.POST)
        # Make this site field required False since you can only update the account_number
        form.fields["site"].required = False

        if not form.is_valid():
            context = super().get_context_data(**kwargs)
            context["form"] = form
            return render(request, self.template_name, context)

        stripe_connect = StripeConnectAccountConfig(self.object.site)
        stripe_connect.save(
            form.cleaned_data["account_number"], "stripe_connect_account"
        )

        return redirect("vendor_admin:manager-config-stripe-connect-list")


class VendorSiteCommissionCreateConfigView(LoginRequiredMixin, FormView):
    template_name = "vendor/manage/config.html"
    form_class = VendorSiteCommissionForm

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        commission_config = VendorSiteCommissionConfig()
        form = self.get_form()

        existing_commission_configs = SiteConfigModel.objects.filter(
            key=commission_config.key
        )
        sites = Site.objects.exclude(
            pk__in=[config.site.pk for config in existing_commission_configs]
        )
        form.fields["site"].queryset = sites

        context["title"] = _("Vendor Site Commission")
        context["form"] = form
        context["cancel_url"] = reverse("vendor_admin:manager-config-commission-list")

        return context

    def form_valid(self, form):
        site = form.cleaned_data["site"]

        commission = VendorSiteCommissionConfig(site)
        commission.save(form.cleaned_data["commission"], "commission")

        return redirect("vendor_admin:manager-config-commission-list")


class VendorSiteCommissionConfigListView(LoginRequiredMixin, ListView):
    template_name = "vendor/manage/config_list.html"
    model = SiteConfigModel

    def get_queryset(self):
        commission_config = VendorSiteCommissionConfig()

        return self.model.objects.filter(key=commission_config.key)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        commission_config = VendorSiteCommissionConfig()

        context["title"] = _("Vendor Commissions")
        context["config_key"] = commission_config.key
        context["new_url"] = reverse("vendor_admin:manager-config-commission-create")

        return context


class VendorSiteCommissionUpdateConfigView(LoginRequiredMixin, UpdateView):
    template_name = "vendor/manage/config.html"
    model = SiteConfigModel
    form_class = VendorSiteCommissionForm

    def get_form(self):
        return VendorSiteCommissionForm()

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        config_model = self.get_object()
        commission_config = VendorSiteCommissionConfig(config_model.site)

        context["title"] = _("Vendor Commission")
        context["form"] = VendorSiteCommissionForm(
            initial={
                "site": config_model.site,
                "commission": commission_config.get_key_value("commission"),
            }
        )
        context["form"].fields["site"].disabled = True
        context["cancel_url"] = reverse("vendor_admin:manager-config-commission-list")

        return context

    def post(self, request, *args, **kwargs):
        # Get the SiteConfigModel instance passed through the url
        self.object = self.get_object()

        form = self.get_form_class()(request.POST)
        # Make this site field required False since you can only update the commission
        form.fields["site"].required = False

        if not form.is_valid():
            context = super().get_context_data(**kwargs)
            context["form"] = form
            return render(request, self.template_name, context)

        stripe_connect = VendorSiteCommissionConfig(self.object.site)
        stripe_connect.save(form.cleaned_data["commission"], "commission")

        return redirect("vendor_admin:manager-config-commission-list")



from django.urls import path

from vendor.api.v1 import views as api_views

app_name = "vendor_api"

urlpatterns = [
    # Shared API Endpoints
    path(
        "cart/<slug:slug>/add/", api_views.AddToCartView.as_view(), name="add-to-cart"
    ),
    path(
        "cart/<slug:slug>/remove/",
        api_views.RemoveFromCartView.as_view(),
        name="remove-from-cart",
    ),
    path(
        "subscription/<uuid:uuid>/cancel/",
        api_views.PaymentGatewaySubscriptionCancelView.as_view(),
        name="customer-subscription-cancel",
    ),
]



# Generated by Django 3.2.19 on 2023-05-09 11:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0042_offer_is_promotional"),
    ]

    operations = [
        migrations.AddField(
            model_name="invoice",
            name="global_discount",
            field=models.FloatField(
                blank=True, default=0, null=True, verbose_name="Global Discount"
            ),
        ),
    ]



from django.db import models
from django.utils.translation import gettext_lazy as _

from .base import CreateUpdateModelBase


###########
# WISHLIST
###########
class Wishlist(models.Model):
    profile = models.ForeignKey(
        "vendor.CustomerProfile",
        verbose_name=_("Purchase Profile"),
        null=True,
        on_delete=models.CASCADE,
        related_name="wishlists",
    )
    name = models.CharField(_("Name"), max_length=100, blank=False)

    class Meta:
        verbose_name = "Wishlist"
        verbose_name_plural = "Wishlists"

    def __str__(self):
        return self.name


################
# WISHLIST ITEM
################
class WishlistItem(CreateUpdateModelBase):
    """ """

    wishlist = models.ForeignKey(
        Wishlist,
        verbose_name=_("Wishlist"),
        on_delete=models.CASCADE,
        related_name="wishlist_items",
    )
    offer = models.ForeignKey(
        "vendor.Offer",
        verbose_name=_("Offer"),
        on_delete=models.CASCADE,
        related_name="wishlist_items",
    )

    class Meta:
        verbose_name = "Wishlist Item"
        verbose_name_plural = "Wishlist Items"
        # TODO: Unique Name Per User

    def __str__(self):
        return "({}) {}: {}".format(
            self.wishlist.profile.user.username, self.wishlist.name, self.offer.name
        )



# Generated by Django 3.1.3 on 2020-11-11 07:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0008_offer_allow_multiple"),
    ]

    operations = [
        migrations.AlterField(
            model_name="offer",
            name="terms",
            field=models.IntegerField(
                choices=[
                    (0, "Perpetual"),
                    (10, "Subscription"),
                    (11, "Monthly Subscription"),
                    (12, "Quarterly Subscription"),
                    (13, "Semi-Annual Subscription"),
                    (14, "Annual Subscription"),
                    (20, "One-Time Use"),
                ],
                default=0,
                verbose_name="Terms",
            ),
        ),
    ]



# Generated by Django 3.1.1 on 2020-09-10 22:06

import uuid

import django.contrib.sites.managers
import django.db.models.deletion
import django.db.models.manager
from django.conf import settings
from django.db import migrations, models

from vendor.config import VENDOR_PRODUCT_MODEL
from vendor.fields import AutoSlugField


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(VENDOR_PRODUCT_MODEL),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.CreateModel(
            name="Address",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        blank=True,
                        default="Home",
                        max_length=80,
                        verbose_name="Address Name",
                    ),
                ),
                (
                    "address_1",
                    models.CharField(max_length=40, verbose_name="Address 1"),
                ),
                (
                    "address_2",
                    models.CharField(
                        blank=True, max_length=40, null=True, verbose_name="Address 2"
                    ),
                ),
                ("locality", models.CharField(max_length=40, verbose_name="City")),
                ("state", models.CharField(max_length=40, verbose_name="State")),
                (
                    "country",
                    models.IntegerField(
                        choices=[(581, "United States")],
                        default=581,
                        verbose_name="Country",
                    ),
                ),
                (
                    "postal_code",
                    models.CharField(
                        blank=True, max_length=16, verbose_name="Postal Code"
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="CustomerProfile",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="date created"
                    ),
                ),
                (
                    "updated",
                    models.DateTimeField(auto_now=True, verbose_name="last updated"),
                ),
                (
                    "currency",
                    models.CharField(
                        choices=[
                            ("afn", "AFN"),
                            ("eur", "EUR"),
                            ("all", "ALL"),
                            ("dzd", "DZD"),
                            ("usd", "USD"),
                            ("aoa", "AOA"),
                            ("xcd", "XCD"),
                            ("ars", "ARS"),
                            ("amd", "AMD"),
                            ("awg", "AWG"),
                            ("aud", "AUD"),
                            ("azn", "AZN"),
                            ("bsd", "BSD"),
                            ("bhd", "BHD"),
                            ("bdt", "BDT"),
                            ("bbd", "BBD"),
                            ("byn", "BYN"),
                            ("bzd", "BZD"),
                            ("xof", "XOF"),
                            ("bmd", "BMD"),
                            ("inr", "INR"),
                            ("btn", "BTN"),
                            ("bob", "BOB"),
                            ("bov", "BOV"),
                            ("bam", "BAM"),
                            ("bwp", "BWP"),
                            ("nok", "NOK"),
                            ("brl", "BRL"),
                            ("bnd", "BND"),
                            ("bgn", "BGN"),
                            ("bif", "BIF"),
                            ("cve", "CVE"),
                            ("khr", "KHR"),
                            ("xaf", "XAF"),
                            ("cad", "CAD"),
                            ("kyd", "KYD"),
                            ("clp", "CLP"),
                            ("clf", "CLF"),
                            ("cny", "CNY"),
                            ("cop", "COP"),
                            ("cou", "COU"),
                            ("kmf", "KMF"),
                            ("cdf", "CDF"),
                            ("nzd", "NZD"),
                            ("crc", "CRC"),
                            ("hrk", "HRK"),
                            ("cup", "CUP"),
                            ("cuc", "CUC"),
                            ("ang", "ANG"),
                            ("czk", "CZK"),
                            ("dkk", "DKK"),
                            ("djf", "DJF"),
                            ("dop", "DOP"),
                            ("egp", "EGP"),
                            ("svc", "SVC"),
                            ("ern", "ERN"),
                            ("etb", "ETB"),
                            ("fkp", "FKP"),
                            ("fjd", "FJD"),
                            ("xpf", "XPF"),
                            ("gmd", "GMD"),
                            ("gel", "GEL"),
                            ("ghs", "GHS"),
                            ("gip", "GIP"),
                            ("gtq", "GTQ"),
                            ("gbp", "GBP"),
                            ("gnf", "GNF"),
                            ("gyd", "GYD"),
                            ("htg", "HTG"),
                            ("hnl", "HNL"),
                            ("hkd", "HKD"),
                            ("huf", "HUF"),
                            ("isk", "ISK"),
                            ("idr", "IDR"),
                            ("irr", "IRR"),
                            ("iqd", "IQD"),
                            ("ils", "ILS"),
                            ("jmd", "JMD"),
                            ("jpy", "JPY"),
                            ("jod", "JOD"),
                            ("kzt", "KZT"),
                            ("kes", "KES"),
                            ("kpw", "KPW"),
                            ("krw", "KRW"),
                            ("kwd", "KWD"),
                            ("kgs", "KGS"),
                            ("lak", "LAK"),
                            ("lbp", "LBP"),
                            ("lsl", "LSL"),
                            ("zar", "ZAR"),
                            ("lrd", "LRD"),
                            ("lyd", "LYD"),
                            ("chf", "CHF"),
                            ("mop", "MOP"),
                            ("mkd", "MKD"),
                            ("mga", "MGA"),
                            ("mwk", "MWK"),
                            ("myr", "MYR"),
                            ("mvr", "MVR"),
                            ("mru", "MRU"),
                            ("mur", "MUR"),
                            ("mxn", "MXN"),
                            ("mxv", "MXV"),
                            ("mdl", "MDL"),
                            ("mnt", "MNT"),
                            ("mad", "MAD"),
                            ("mzn", "MZN"),
                            ("mmk", "MMK"),
                            ("nad", "NAD"),
                            ("npr", "NPR"),
                            ("nio", "NIO"),
                            ("ngn", "NGN"),
                            ("omr", "OMR"),
                            ("pkr", "PKR"),
                            ("pab", "PAB"),
                            ("pgk", "PGK"),
                            ("pyg", "PYG"),
                            ("pen", "PEN"),
                            ("php", "PHP"),
                            ("pln", "PLN"),
                            ("qar", "QAR"),
                            ("ron", "RON"),
                            ("rub", "RUB"),
                            ("rwf", "RWF"),
                            ("shp", "SHP"),
                            ("wst", "WST"),
                            ("stn", "STN"),
                            ("sar", "SAR"),
                            ("rsd", "RSD"),
                            ("scr", "SCR"),
                            ("sll", "SLL"),
                            ("sgd", "SGD"),
                            ("sbd", "SBD"),
                            ("sos", "SOS"),
                            ("ssp", "SSP"),
                            ("lkr", "LKR"),
                            ("sdg", "SDG"),
                            ("srd", "SRD"),
                            ("szl", "SZL"),
                            ("sek", "SEK"),
                            ("che", "CHE"),
                            ("chw", "CHW"),
                            ("syp", "SYP"),
                            ("twd", "TWD"),
                            ("tjs", "TJS"),
                            ("tzs", "TZS"),
                            ("thb", "THB"),
                            ("top", "TOP"),
                            ("ttd", "TTD"),
                            ("tnd", "TND"),
                            ("try", "TRY"),
                            ("tmt", "TMT"),
                            ("ugx", "UGX"),
                            ("uah", "UAH"),
                            ("aed", "AED"),
                            ("usn", "USN"),
                            ("uyu", "UYU"),
                            ("uyi", "UYI"),
                            ("uyw", "UYW"),
                            ("uzs", "UZS"),
                            ("vuv", "VUV"),
                            ("ves", "VES"),
                            ("vnd", "VND"),
                            ("yer", "YER"),
                            ("zmw", "ZMW"),
                            ("zwl", "ZWL"),
                        ],
                        default="usd",
                        max_length=4,
                        verbose_name="Currency",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        default=1,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="customer_profile",
                        to="sites.site",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="customer_profile",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="User",
                    ),
                ),
            ],
            options={
                "verbose_name": "Customer Profile",
                "verbose_name_plural": "Customer Profiles",
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name="Invoice",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="date created"
                    ),
                ),
                (
                    "updated",
                    models.DateTimeField(auto_now=True, verbose_name="last updated"),
                ),
                (
                    "uuid",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        unique=True,
                        verbose_name="UUID",
                    ),
                ),
                (
                    "status",
                    models.IntegerField(
                        choices=[
                            (0, "Cart"),
                            (10, "Checkout"),
                            (20, "Queued"),
                            (30, "Processing"),
                            (40, "Failed"),
                            (50, "Complete"),
                            (60, "Refunded"),
                        ],
                        default=0,
                        verbose_name="Status",
                    ),
                ),
                (
                    "customer_notes",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        null=True,
                        verbose_name="Customer Notes",
                    ),
                ),
                (
                    "vendor_notes",
                    models.JSONField(
                        blank=True, default=dict, null=True, verbose_name="Vendor Notes"
                    ),
                ),
                (
                    "ordered_date",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="Ordered Date"
                    ),
                ),
                ("subtotal", models.FloatField(default=0.0)),
                ("tax", models.FloatField(blank=True, null=True)),
                ("shipping", models.FloatField(blank=True, null=True)),
                ("total", models.FloatField(blank=True, null=True)),
                (
                    "currency",
                    models.CharField(
                        choices=[
                            ("afn", "AFN"),
                            ("eur", "EUR"),
                            ("all", "ALL"),
                            ("dzd", "DZD"),
                            ("usd", "USD"),
                            ("aoa", "AOA"),
                            ("xcd", "XCD"),
                            ("ars", "ARS"),
                            ("amd", "AMD"),
                            ("awg", "AWG"),
                            ("aud", "AUD"),
                            ("azn", "AZN"),
                            ("bsd", "BSD"),
                            ("bhd", "BHD"),
                            ("bdt", "BDT"),
                            ("bbd", "BBD"),
                            ("byn", "BYN"),
                            ("bzd", "BZD"),
                            ("xof", "XOF"),
                            ("bmd", "BMD"),
                            ("inr", "INR"),
                            ("btn", "BTN"),
                            ("bob", "BOB"),
                            ("bov", "BOV"),
                            ("bam", "BAM"),
                            ("bwp", "BWP"),
                            ("nok", "NOK"),
                            ("brl", "BRL"),
                            ("bnd", "BND"),
                            ("bgn", "BGN"),
                            ("bif", "BIF"),
                            ("cve", "CVE"),
                            ("khr", "KHR"),
                            ("xaf", "XAF"),
                            ("cad", "CAD"),
                            ("kyd", "KYD"),
                            ("clp", "CLP"),
                            ("clf", "CLF"),
                            ("cny", "CNY"),
                            ("cop", "COP"),
                            ("cou", "COU"),
                            ("kmf", "KMF"),
                            ("cdf", "CDF"),
                            ("nzd", "NZD"),
                            ("crc", "CRC"),
                            ("hrk", "HRK"),
                            ("cup", "CUP"),
                            ("cuc", "CUC"),
                            ("ang", "ANG"),
                            ("czk", "CZK"),
                            ("dkk", "DKK"),
                            ("djf", "DJF"),
                            ("dop", "DOP"),
                            ("egp", "EGP"),
                            ("svc", "SVC"),
                            ("ern", "ERN"),
                            ("etb", "ETB"),
                            ("fkp", "FKP"),
                            ("fjd", "FJD"),
                            ("xpf", "XPF"),
                            ("gmd", "GMD"),
                            ("gel", "GEL"),
                            ("ghs", "GHS"),
                            ("gip", "GIP"),
                            ("gtq", "GTQ"),
                            ("gbp", "GBP"),
                            ("gnf", "GNF"),
                            ("gyd", "GYD"),
                            ("htg", "HTG"),
                            ("hnl", "HNL"),
                            ("hkd", "HKD"),
                            ("huf", "HUF"),
                            ("isk", "ISK"),
                            ("idr", "IDR"),
                            ("irr", "IRR"),
                            ("iqd", "IQD"),
                            ("ils", "ILS"),
                            ("jmd", "JMD"),
                            ("jpy", "JPY"),
                            ("jod", "JOD"),
                            ("kzt", "KZT"),
                            ("kes", "KES"),
                            ("kpw", "KPW"),
                            ("krw", "KRW"),
                            ("kwd", "KWD"),
                            ("kgs", "KGS"),
                            ("lak", "LAK"),
                            ("lbp", "LBP"),
                            ("lsl", "LSL"),
                            ("zar", "ZAR"),
                            ("lrd", "LRD"),
                            ("lyd", "LYD"),
                            ("chf", "CHF"),
                            ("mop", "MOP"),
                            ("mkd", "MKD"),
                            ("mga", "MGA"),
                            ("mwk", "MWK"),
                            ("myr", "MYR"),
                            ("mvr", "MVR"),
                            ("mru", "MRU"),
                            ("mur", "MUR"),
                            ("mxn", "MXN"),
                            ("mxv", "MXV"),
                            ("mdl", "MDL"),
                            ("mnt", "MNT"),
                            ("mad", "MAD"),
                            ("mzn", "MZN"),
                            ("mmk", "MMK"),
                            ("nad", "NAD"),
                            ("npr", "NPR"),
                            ("nio", "NIO"),
                            ("ngn", "NGN"),
                            ("omr", "OMR"),
                            ("pkr", "PKR"),
                            ("pab", "PAB"),
                            ("pgk", "PGK"),
                            ("pyg", "PYG"),
                            ("pen", "PEN"),
                            ("php", "PHP"),
                            ("pln", "PLN"),
                            ("qar", "QAR"),
                            ("ron", "RON"),
                            ("rub", "RUB"),
                            ("rwf", "RWF"),
                            ("shp", "SHP"),
                            ("wst", "WST"),
                            ("stn", "STN"),
                            ("sar", "SAR"),
                            ("rsd", "RSD"),
                            ("scr", "SCR"),
                            ("sll", "SLL"),
                            ("sgd", "SGD"),
                            ("sbd", "SBD"),
                            ("sos", "SOS"),
                            ("ssp", "SSP"),
                            ("lkr", "LKR"),
                            ("sdg", "SDG"),
                            ("srd", "SRD"),
                            ("szl", "SZL"),
                            ("sek", "SEK"),
                            ("che", "CHE"),
                            ("chw", "CHW"),
                            ("syp", "SYP"),
                            ("twd", "TWD"),
                            ("tjs", "TJS"),
                            ("tzs", "TZS"),
                            ("thb", "THB"),
                            ("top", "TOP"),
                            ("ttd", "TTD"),
                            ("tnd", "TND"),
                            ("try", "TRY"),
                            ("tmt", "TMT"),
                            ("ugx", "UGX"),
                            ("uah", "UAH"),
                            ("aed", "AED"),
                            ("usn", "USN"),
                            ("uyu", "UYU"),
                            ("uyi", "UYI"),
                            ("uyw", "UYW"),
                            ("uzs", "UZS"),
                            ("vuv", "VUV"),
                            ("ves", "VES"),
                            ("vnd", "VND"),
                            ("yer", "YER"),
                            ("zmw", "ZMW"),
                            ("zwl", "ZWL"),
                        ],
                        default="usd",
                        max_length=4,
                        verbose_name="Currency",
                    ),
                ),
                (
                    "profile",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="invoices",
                        to="vendor.customerprofile",
                        verbose_name="Customer Profile",
                    ),
                ),
                (
                    "shipping_address",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="vendor.address",
                        verbose_name="Shipping Address",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        default=1,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="invoices",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Invoice",
                "verbose_name_plural": "Invoices",
                "ordering": ["-ordered_date", "-updated"],
                "permissions": (
                    ("can_view_site_purchases", "Can view Site Purchases"),
                    ("can_refund_purchase", "Can refund Purchase"),
                ),
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name="Offer",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="date created"
                    ),
                ),
                (
                    "updated",
                    models.DateTimeField(auto_now=True, verbose_name="last updated"),
                ),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "slug",
                    AutoSlugField(
                        editable=False, populate_from="name", unique_with=("site__id",)
                    ),
                ),
                (
                    "name",
                    models.CharField(blank=True, max_length=80, verbose_name="Name"),
                ),
                (
                    "start_date",
                    models.DateTimeField(
                        help_text="What date should this offer become available?",
                        verbose_name="Start Date",
                    ),
                ),
                (
                    "end_date",
                    models.DateTimeField(
                        blank=True,
                        help_text="Expiration Date?",
                        null=True,
                        verbose_name="End Date",
                    ),
                ),
                (
                    "terms",
                    models.IntegerField(
                        choices=[
                            (0, "Perpetual"),
                            (10, "Subscription"),
                            (20, "One-Time Use"),
                        ],
                        default=0,
                        verbose_name="Terms",
                    ),
                ),
                (
                    "term_details",
                    models.JSONField(
                        blank=True, default=dict, null=True, verbose_name="Term Details"
                    ),
                ),
                (
                    "term_start_date",
                    models.DateTimeField(
                        blank=True,
                        help_text="When is this product available to use?",
                        null=True,
                        verbose_name="Term Start Date",
                    ),
                ),
                (
                    "available",
                    models.BooleanField(
                        default=False,
                        help_text="Is this currently available?",
                        verbose_name="Available",
                    ),
                ),
                (
                    "bundle",
                    models.BooleanField(
                        default=False,
                        help_text="Is this a product bundle? (auto-generated)",
                        verbose_name="Is a Bundle?",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        default=1,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="product_offers",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Offer",
                "verbose_name_plural": "Offers",
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name="OrderItem",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="date created"
                    ),
                ),
                (
                    "updated",
                    models.DateTimeField(auto_now=True, verbose_name="last updated"),
                ),
                ("quantity", models.IntegerField(default=1, verbose_name="Quantity")),
                (
                    "invoice",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="order_items",
                        to="vendor.invoice",
                        verbose_name="Invoice",
                    ),
                ),
                (
                    "offer",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="order_items",
                        to="vendor.offer",
                        verbose_name="Offer",
                    ),
                ),
            ],
            options={
                "verbose_name": "Order Item",
                "verbose_name_plural": "Order Items",
            },
        ),
        migrations.CreateModel(
            name="TaxClassifier",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(blank=True, max_length=80, verbose_name="Name"),
                ),
                ("taxable", models.BooleanField()),
            ],
            options={
                "verbose_name": "Product Classifier",
                "verbose_name_plural": "Product Classifiers",
            },
        ),
        migrations.CreateModel(
            name="Wishlist",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, verbose_name="Name")),
                (
                    "profile",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="wishlists",
                        to="vendor.customerprofile",
                        verbose_name="Purchase Profile",
                    ),
                ),
            ],
            options={
                "verbose_name": "Wishlist",
                "verbose_name_plural": "Wishlists",
            },
        ),
        migrations.CreateModel(
            name="WishlistItem",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="date created"
                    ),
                ),
                (
                    "updated",
                    models.DateTimeField(auto_now=True, verbose_name="last updated"),
                ),
                (
                    "offer",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="wishlist_items",
                        to="vendor.offer",
                        verbose_name="Offer",
                    ),
                ),
                (
                    "wishlist",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="wishlist_items",
                        to="vendor.wishlist",
                        verbose_name="Wishlist",
                    ),
                ),
            ],
            options={
                "verbose_name": "Wishlist Item",
                "verbose_name_plural": "Wishlist Items",
            },
        ),
        migrations.CreateModel(
            name="Receipt",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="date created"
                    ),
                ),
                (
                    "updated",
                    models.DateTimeField(auto_now=True, verbose_name="last updated"),
                ),
                (
                    "start_date",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="Start Date"
                    ),
                ),
                (
                    "end_date",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="End Date"
                    ),
                ),
                (
                    "auto_renew",
                    models.BooleanField(default=False, verbose_name="Auto Renew"),
                ),
                (
                    "vendor_notes",
                    models.JSONField(default=dict, verbose_name="Vendor Notes"),
                ),
                (
                    "transaction",
                    models.CharField(max_length=80, verbose_name="Transaction"),
                ),
                (
                    "status",
                    models.IntegerField(
                        choices=[
                            (1, "Queued"),
                            (2, "Active"),
                            (10, "Authorized"),
                            (15, "Captured"),
                            (20, "Completed"),
                            (30, "Canceled"),
                            (35, "Refunded"),
                        ],
                        default=0,
                        verbose_name="Status",
                    ),
                ),
                ("meta", models.JSONField(default=dict)),
                (
                    "order_item",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="receipts",
                        to="vendor.orderitem",
                        verbose_name="Order Item",
                    ),
                ),
                (
                    "profile",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="receipts",
                        to="vendor.customerprofile",
                        verbose_name="Purchase Profile",
                    ),
                ),
            ],
            options={
                "verbose_name": "Receipt",
                "verbose_name_plural": "Receipts",
            },
        ),
        migrations.CreateModel(
            name="Price",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("cost", models.FloatField(blank=True, null=True)),
                (
                    "currency",
                    models.CharField(
                        choices=[
                            ("afn", "AFN"),
                            ("eur", "EUR"),
                            ("all", "ALL"),
                            ("dzd", "DZD"),
                            ("usd", "USD"),
                            ("aoa", "AOA"),
                            ("xcd", "XCD"),
                            ("ars", "ARS"),
                            ("amd", "AMD"),
                            ("awg", "AWG"),
                            ("aud", "AUD"),
                            ("azn", "AZN"),
                            ("bsd", "BSD"),
                            ("bhd", "BHD"),
                            ("bdt", "BDT"),
                            ("bbd", "BBD"),
                            ("byn", "BYN"),
                            ("bzd", "BZD"),
                            ("xof", "XOF"),
                            ("bmd", "BMD"),
                            ("inr", "INR"),
                            ("btn", "BTN"),
                            ("bob", "BOB"),
                            ("bov", "BOV"),
                            ("bam", "BAM"),
                            ("bwp", "BWP"),
                            ("nok", "NOK"),
                            ("brl", "BRL"),
                            ("bnd", "BND"),
                            ("bgn", "BGN"),
                            ("bif", "BIF"),
                            ("cve", "CVE"),
                            ("khr", "KHR"),
                            ("xaf", "XAF"),
                            ("cad", "CAD"),
                            ("kyd", "KYD"),
                            ("clp", "CLP"),
                            ("clf", "CLF"),
                            ("cny", "CNY"),
                            ("cop", "COP"),
                            ("cou", "COU"),
                            ("kmf", "KMF"),
                            ("cdf", "CDF"),
                            ("nzd", "NZD"),
                            ("crc", "CRC"),
                            ("hrk", "HRK"),
                            ("cup", "CUP"),
                            ("cuc", "CUC"),
                            ("ang", "ANG"),
                            ("czk", "CZK"),
                            ("dkk", "DKK"),
                            ("djf", "DJF"),
                            ("dop", "DOP"),
                            ("egp", "EGP"),
                            ("svc", "SVC"),
                            ("ern", "ERN"),
                            ("etb", "ETB"),
                            ("fkp", "FKP"),
                            ("fjd", "FJD"),
                            ("xpf", "XPF"),
                            ("gmd", "GMD"),
                            ("gel", "GEL"),
                            ("ghs", "GHS"),
                            ("gip", "GIP"),
                            ("gtq", "GTQ"),
                            ("gbp", "GBP"),
                            ("gnf", "GNF"),
                            ("gyd", "GYD"),
                            ("htg", "HTG"),
                            ("hnl", "HNL"),
                            ("hkd", "HKD"),
                            ("huf", "HUF"),
                            ("isk", "ISK"),
                            ("idr", "IDR"),
                            ("irr", "IRR"),
                            ("iqd", "IQD"),
                            ("ils", "ILS"),
                            ("jmd", "JMD"),
                            ("jpy", "JPY"),
                            ("jod", "JOD"),
                            ("kzt", "KZT"),
                            ("kes", "KES"),
                            ("kpw", "KPW"),
                            ("krw", "KRW"),
                            ("kwd", "KWD"),
                            ("kgs", "KGS"),
                            ("lak", "LAK"),
                            ("lbp", "LBP"),
                            ("lsl", "LSL"),
                            ("zar", "ZAR"),
                            ("lrd", "LRD"),
                            ("lyd", "LYD"),
                            ("chf", "CHF"),
                            ("mop", "MOP"),
                            ("mkd", "MKD"),
                            ("mga", "MGA"),
                            ("mwk", "MWK"),
                            ("myr", "MYR"),
                            ("mvr", "MVR"),
                            ("mru", "MRU"),
                            ("mur", "MUR"),
                            ("mxn", "MXN"),
                            ("mxv", "MXV"),
                            ("mdl", "MDL"),
                            ("mnt", "MNT"),
                            ("mad", "MAD"),
                            ("mzn", "MZN"),
                            ("mmk", "MMK"),
                            ("nad", "NAD"),
                            ("npr", "NPR"),
                            ("nio", "NIO"),
                            ("ngn", "NGN"),
                            ("omr", "OMR"),
                            ("pkr", "PKR"),
                            ("pab", "PAB"),
                            ("pgk", "PGK"),
                            ("pyg", "PYG"),
                            ("pen", "PEN"),
                            ("php", "PHP"),
                            ("pln", "PLN"),
                            ("qar", "QAR"),
                            ("ron", "RON"),
                            ("rub", "RUB"),
                            ("rwf", "RWF"),
                            ("shp", "SHP"),
                            ("wst", "WST"),
                            ("stn", "STN"),
                            ("sar", "SAR"),
                            ("rsd", "RSD"),
                            ("scr", "SCR"),
                            ("sll", "SLL"),
                            ("sgd", "SGD"),
                            ("sbd", "SBD"),
                            ("sos", "SOS"),
                            ("ssp", "SSP"),
                            ("lkr", "LKR"),
                            ("sdg", "SDG"),
                            ("srd", "SRD"),
                            ("szl", "SZL"),
                            ("sek", "SEK"),
                            ("che", "CHE"),
                            ("chw", "CHW"),
                            ("syp", "SYP"),
                            ("twd", "TWD"),
                            ("tjs", "TJS"),
                            ("tzs", "TZS"),
                            ("thb", "THB"),
                            ("top", "TOP"),
                            ("ttd", "TTD"),
                            ("tnd", "TND"),
                            ("try", "TRY"),
                            ("tmt", "TMT"),
                            ("ugx", "UGX"),
                            ("uah", "UAH"),
                            ("aed", "AED"),
                            ("usn", "USN"),
                            ("uyu", "UYU"),
                            ("uyi", "UYI"),
                            ("uyw", "UYW"),
                            ("uzs", "UZS"),
                            ("vuv", "VUV"),
                            ("ves", "VES"),
                            ("vnd", "VND"),
                            ("yer", "YER"),
                            ("zmw", "ZMW"),
                            ("zwl", "ZWL"),
                        ],
                        default="usd",
                        max_length=4,
                        verbose_name="Currency",
                    ),
                ),
                (
                    "start_date",
                    models.DateTimeField(
                        help_text="When should the price first become available?",
                        verbose_name="Start Date",
                    ),
                ),
                (
                    "end_date",
                    models.DateTimeField(
                        blank=True,
                        help_text="When should the price expire?",
                        null=True,
                        verbose_name="End Date",
                    ),
                ),
                (
                    "priority",
                    models.IntegerField(
                        blank=True,
                        help_text="Higher number takes priority",
                        null=True,
                        verbose_name="Priority",
                    ),
                ),
                (
                    "offer",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="prices",
                        to="vendor.offer",
                    ),
                ),
            ],
            options={
                "verbose_name": "Price",
                "verbose_name_plural": "Prices",
            },
        ),
        migrations.CreateModel(
            name="Payment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="date created"
                    ),
                ),
                (
                    "transaction",
                    models.CharField(max_length=50, verbose_name="Transaction ID"),
                ),
                (
                    "provider",
                    models.CharField(max_length=30, verbose_name="Payment Provider"),
                ),
                ("amount", models.FloatField(verbose_name="Amount")),
                (
                    "result",
                    models.JSONField(
                        blank=True, default=dict, null=True, verbose_name="Result"
                    ),
                ),
                (
                    "success",
                    models.BooleanField(default=False, verbose_name="Successful"),
                ),
                (
                    "payee_full_name",
                    models.CharField(max_length=50, verbose_name="Name on Card"),
                ),
                (
                    "payee_company",
                    models.CharField(
                        blank=True, max_length=50, null=True, verbose_name="Company"
                    ),
                ),
                (
                    "billing_address",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="vendor.address",
                        verbose_name="payments",
                    ),
                ),
                (
                    "invoice",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="payments",
                        to="vendor.invoice",
                        verbose_name="Invoice",
                    ),
                ),
                (
                    "profile",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="payments",
                        to="vendor.customerprofile",
                        verbose_name="Purchase Profile",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="address",
            name="profile",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="addresses",
                to="vendor.customerprofile",
                verbose_name="Customer Profile",
            ),
        ),
    ]



from django.contrib.sites.managers import CurrentSiteManager
from django.db import models


class ActiveManager(models.Manager):
    """
    This Model Manger returns offers that are available
    """

    def get_queryset(self):
        return super().get_queryset().filter(available=True)


class ActiveCurrentSiteManager(CurrentSiteManager):
    """
    This Model Manager return offers per site that are available
    """

    def get_queryset(self):
        return super().get_queryset().filter(available=True)


class SoftDeleteManager(models.Manager):

    def get_queryset(self):
        return super().get_queryset().filter(deleted=False)


class CurrentSiteSoftDeleteManager(CurrentSiteManager):

    def get_queryset(self):
        return super().get_queryset().filter(deleted=False)



# Generated by Django 3.2.13 on 2022-06-01 18:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("vendor", "0031_pre_invoice_status_change"),
    ]

    operations = [
        migrations.AlterField(
            model_name="invoice",
            name="status",
            field=models.IntegerField(
                choices=[(0, "Cart"), (10, "Checkout"), (20, "Complete")],
                default=0,
                verbose_name="Status",
            ),
        ),
    ]
